@@ -19,6 +19,7 @@ import os
 from os.path import dirname
 import sys
 
+import bzrlib.bzrdir
 import bzrlib.errors as errors
 from bzrlib.inventory import InventoryEntry
 from bzrlib.trace import mutter, note, warning
@@ -168,6 +169,14 @@ class FastPath(object):
 
 
 def smart_add_tree(tree, file_list, recurse=True, action=None, save=True):
+    tree.lock_tree_write()
+    try:
+        return _smart_add_tree(tree=tree, file_list=file_list, recurse=recurse,
+                               action=action, save=save)
+    finally:
+        tree.unlock()
+
+def _smart_add_tree(tree, file_list, recurse=True, action=None, save=True):
     """Add files to version, optionally recursing into directories.
 
     This is designed more towards DWIM for humans than API simplicity.
@@ -203,7 +212,7 @@ def smart_add_tree(tree, file_list, recurse=True, action=None, save=True):
         # validate user parameters. Our recursive code avoids adding new files
         # that need such validation 
         if tree.is_control_filename(rf.raw_path):
-            raise errors.ForbiddenControlFileError(filename=rf)
+            raise errors.ForbiddenControlFileError(filename=rf.raw_path)
         
         abspath = tree.abspath(rf.raw_path)
         kind = bzrlib.osutils.file_kind(abspath)
@@ -278,6 +287,13 @@ def smart_add_tree(tree, file_list, recurse=True, action=None, save=True):
             pass
             # mutter("%r is already versioned", abspath)
         elif sub_tree:
+            # XXX: This is wrong; people *might* reasonably be trying to add
+            # subtrees as subtrees.  This should probably only be done in formats 
+            # which can represent subtrees, and even then perhaps only when
+            # the user asked to add subtrees.  At the moment you can add them
+            # specially through 'join --reference', which is perhaps
+            # reasonable: adding a new reference is a special operation and
+            # can have a special behaviour.  mbp 20070306
             mutter("%r is a nested bzr tree", abspath)
         else:
             __add_one(tree, inv, parent_ie, directory, kind, action)
