@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005 Canonical Ltd
+# Copyright (C) 2004, 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -83,18 +83,17 @@ def annotate_file(branch, rev_id, file_id, verbose=False, full=False,
         prevanno=anno
 
 
-def _annotate_file(branch, rev_id, file_id ):
+def _annotate_file(branch, rev_id, file_id):
     """Yield the origins for each line of a file.
 
     This includes detailed information, such as the committer name, and
     date string for the commit, rather than just the revision id.
     """
-
-    rh = branch.revision_history()
-    revision_graph = branch.repository.get_revision_graph(rev_id)
+    branch_last_revision = branch.last_revision()
+    revision_graph = branch.repository.get_revision_graph(branch_last_revision)
     merge_sorted_revisions = tsort.merge_sort(
         revision_graph,
-        rev_id,
+        branch_last_revision,
         None,
         generate_revno=True)
     revision_id_to_revno = dict((rev_id, revno)
