@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,15 +15,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-# XXX: Can we do any better about making interrupted commits change
-# nothing?  
-
-# TODO: Separate 'prepare' phase where we find a list of potentially
-# committed files.  We then can then pause the commit to prompt for a
-# commit message, knowing the summary will be the same as what's
-# actually used for the commit.  (But perhaps simpler to simply get
-# the tree status, then use that for a selective commit?)
-
 # The newly committed revision is going to have a shape corresponding
 # to that of the working inventory.  Files that are not in the
 # working tree and that were in the predecessor are reported as
@@ -55,12 +46,6 @@
 # merges from, then it should still be reported as newly added
 # relative to the basis revision.
 
-# TODO: Do checks that the tree can be committed *before* running the 
-# editor; this should include checks for a pointless commit and for 
-# unknown or missing files.
-
-# TODO: If commit fails, leave the message in a file somewhere.
-
 # TODO: Change the parameter 'rev_id' to 'revision_id' to be consistent with
 # the rest of the code; add a deprecation of the old name.
 
@@ -73,8 +58,10 @@ from cStringIO import StringIO
 
 from bzrlib import (
     errors,
+    inventory,
     tree,
     )
+from bzrlib.branch import Branch
 import bzrlib.config
 from bzrlib.errors import (BzrError, PointlessCommit,
                            ConflictsInTree,
@@ -86,12 +73,13 @@ from bzrlib.osutils import (kind_marker, isdir,isfile, is_inside_any,
 from bzrlib.testament import Testament
 from bzrlib.trace import mutter, note, warning
 from bzrlib.xml5 import serializer_v5
-from bzrlib.inventory import Inventory, ROOT_ID, InventoryEntry
+from bzrlib.inventory import Inventory, InventoryEntry
 from bzrlib import symbol_versioning
 from bzrlib.symbol_versioning import (deprecated_passed,
         deprecated_function,
         DEPRECATED_PARAMETER)
 from bzrlib.workingtree import WorkingTree
+import bzrlib.ui
 
 
 class NullCommitReporter(object):
@@ -126,6 +114,8 @@ class ReportCommitToLog(NullCommitReporter):
     def snapshot_change(self, change, path):
         if change == 'unchanged':
             return
+        if change == 'added' and path == '':
+            return
         note("%s %s", change, path)
 
     def completed(self, revno, rev_id):
@@ -182,13 +172,15 @@ class Commit(object):
                working_tree=None,
                local=False,
                reporter=None,
-               config=None):
+               config=None,
+               message_callback=None,
+               recursive='down'):
         """Commit working copy as a new revision.
 
         branch -- the deprecated branch to commit to. New callers should pass in 
                   working_tree instead
 
-        message -- the commit message, a mandatory parameter
+        message -- the commit message (it or message_callback is required)
 
         timestamp -- if not None, seconds-since-epoch for a
              postdated/predated commit.
@@ -209,6 +201,8 @@ class Commit(object):
 
         revprops -- Properties for new revision
         :param local: Perform a local only commit.
+        :param recursive: If set to 'down', commit in any subtrees that have
+            pending changes of any sort during this commit.
         """
         mutter('preparing to commit')
 
@@ -223,8 +217,17 @@ class Commit(object):
         else:
             self.work_tree = working_tree
             self.branch = self.work_tree.branch
-        if message is None:
-            raise BzrError("The message keyword parameter is required for commit().")
+            if getattr(self.work_tree, 'requires_rich_root', lambda: False)():
+                if not self.branch.repository.supports_rich_root():
+                    raise errors.RootNotRich()
+        if message_callback is None:
+            if message is not None:
+                if isinstance(message, str):
+                    message = message.decode(bzrlib.user_encoding)
+                message_callback = lambda x: message
+            else:
+                raise BzrError("The message or message_callback keyword"
+                               " parameter is required for commit().")
 
         self.bound_branch = None
         self.local = local
@@ -233,6 +236,16 @@ class Commit(object):
         self.rev_id = None
         self.specific_files = specific_files
         self.allow_pointless = allow_pointless
+        self.recursive = recursive
+        self.revprops = revprops
+        self.message_callback = message_callback
+        self.timestamp = timestamp
+        self.timezone = timezone
+        self.committer = committer
+        self.specific_files = specific_files
+        self.strict = strict
+        self.verbose = verbose
+        self.local = local
 
         if reporter is None and self.reporter is None:
             self.reporter = NullCommitReporter()
@@ -241,6 +254,8 @@ class Commit(object):
 
         self.work_tree.lock_write()
         self.pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        self.basis_tree = self.work_tree.basis_tree()
+        self.basis_tree.lock_read()
         try:
             # Cannot commit with conflicts present.
             if len(self.work_tree.conflicts())>0:
@@ -257,12 +272,16 @@ class Commit(object):
                 # this is so that we still consier the master branch
                 # - in a checkout scenario the tree may have no
                 # parents but the branch may do.
-                first_tree_parent = None
-            master_last = self.master_branch.last_revision()
-            if (master_last is not None and
-                master_last != first_tree_parent):
-                raise errors.OutOfDateTree(self.work_tree)
-    
+                first_tree_parent = bzrlib.revision.NULL_REVISION
+            old_revno, master_last = self.master_branch.last_revision_info()
+            if master_last != first_tree_parent:
+                if master_last != bzrlib.revision.NULL_REVISION:
+                    raise errors.OutOfDateTree(self.work_tree)
+            if self.branch.repository.has_revision(first_tree_parent):
+                new_revno = old_revno + 1
+            else:
+                # ghost parents never appear in revision history.
+                new_revno = 1
             if strict:
                 # raise an exception as soon as we find a single unknown.
                 for unknown in self.work_tree.unknowns():
@@ -270,20 +289,15 @@ class Commit(object):
                    
             if self.config is None:
                 self.config = self.branch.get_config()
-      
-            if isinstance(message, str):
-                message = message.decode(bzrlib.user_encoding)
-            assert isinstance(message, unicode), type(message)
-            self.message = message
-            self._escape_commit_message()
 
             self.work_inv = self.work_tree.inventory
-            self.basis_tree = self.work_tree.basis_tree()
             self.basis_inv = self.basis_tree.inventory
             if specific_files is not None:
                 # Ensure specified files are versioned
                 # (We don't actually need the ids here)
-                tree.find_ids_across_trees(specific_files, 
+                # XXX: Dont we have filter_unversioned to do this more
+                # cheaply?
+                tree.find_ids_across_trees(specific_files,
                                            [self.basis_tree, self.work_tree])
             # one to finish, one for rev and inventory, and one for each
             # inventory entry, and the same for the new inventory.
@@ -298,7 +312,7 @@ class Commit(object):
                 raise NotImplementedError('selected-file commit of merges is not supported yet: files %r',
                         self.specific_files)
             
-            self.builder = self.branch.get_commit_builder(self.parents, 
+            self.builder = self.branch.get_commit_builder(self.parents,
                 self.config, timestamp, timezone, committer, revprops, rev_id)
             
             self._remove_deleted()
@@ -315,6 +329,11 @@ class Commit(object):
             # that commit will succeed.
             self.builder.finish_inventory()
             self._emit_progress_update()
+            message = message_callback(self)
+            assert isinstance(message, unicode), type(message)
+            self.message = message
+            self._escape_commit_message()
+
             self.rev_id = self.builder.commit(self.message)
             self._emit_progress_update()
             # revision data is in the local branch now.
@@ -327,16 +346,19 @@ class Commit(object):
                 # now the master has the revision data
                 # 'commit' to the master first so a timeout here causes the local
                 # branch to be out of date
-                self.master_branch.append_revision(self.rev_id)
+                self.master_branch.set_last_revision_info(new_revno,
+                                                          self.rev_id)
 
             # and now do the commit locally.
-            self.branch.append_revision(self.rev_id)
+            self.branch.set_last_revision_info(new_revno, self.rev_id)
 
             rev_tree = self.builder.revision_tree()
             self.work_tree.set_parent_trees([(self.rev_id, rev_tree)])
             # now the work tree is up to date with the branch
             
-            self.reporter.completed(self.branch.revno(), self.rev_id)
+            self.reporter.completed(new_revno, self.rev_id)
+            # old style commit hooks - should be deprecated ? (obsoleted in
+            # 0.15)
             if self.config.post_commit() is not None:
                 hooks = self.config.post_commit().split(' ')
                 # this would be nicer with twisted.python.reflect.namedAny
@@ -345,6 +367,23 @@ class Commit(object):
                                   {'branch':self.branch,
                                    'bzrlib':bzrlib,
                                    'rev_id':self.rev_id})
+            # new style commit hooks:
+            if not self.bound_branch:
+                hook_master = self.branch
+                hook_local = None
+            else:
+                hook_master = self.master_branch
+                hook_local = self.branch
+            # With bound branches, when the master is behind the local branch,
+            # the 'old_revno' and old_revid values here are incorrect.
+            # XXX: FIXME ^. RBC 20060206
+            if self.parents:
+                old_revid = self.parents[0]
+            else:
+                old_revid = bzrlib.revision.NULL_REVISION
+            for hook in Branch.hooks['post_commit']:
+                hook(hook_local, hook_master, old_revno, old_revid, new_revno,
+                    self.rev_id)
             self._emit_progress_update()
         finally:
             self._cleanup()
@@ -373,6 +412,7 @@ class Commit(object):
                     and (this.parent_id == other.parent_id)
                     and (this.kind == other.kind)
                     and (this.executable == other.executable)
+                    and (this.reference_revision == other.reference_revision)
                     )
         if not ie_equal_no_revision(new_root_ie, basis_root_ie):
             return True
@@ -390,6 +430,10 @@ class Commit(object):
         # A merge with no effect on files
         if len(self.parents) > 1:
             return
+        # work around the fact that a newly-initted tree does differ from its
+        # basis
+        if len(self.basis_inv) == 0 and len(self.builder.new_inventory) == 1:
+            raise PointlessCommit()
         # Shortcut, if the number of entries changes, then we obviously have
         # a change
         if len(self.builder.new_inventory) != len(self.basis_inv):
@@ -430,9 +474,9 @@ class Commit(object):
         #       to local.
         
         # Make sure the local branch is identical to the master
-        master_rh = self.master_branch.revision_history()
-        local_rh = self.branch.revision_history()
-        if local_rh != master_rh:
+        master_info = self.master_branch.last_revision_info()
+        local_info = self.branch.last_revision_info()
+        if local_info != master_info:
             raise errors.BoundBranchOutOfDate(self.branch,
                     self.master_branch)
 
@@ -445,6 +489,7 @@ class Commit(object):
     def _cleanup(self):
         """Cleanup any open locks, progress bars etc."""
         cleanups = [self._cleanup_bound_branch,
+                    self.basis_tree.unlock,
                     self.work_tree.unlock,
                     self.pb.finished]
         found_exception = None
@@ -500,8 +545,8 @@ class Commit(object):
         # TODO: Make sure that this list doesn't contain duplicate 
         # entries and the order is preserved when doing this.
         self.parents = self.work_tree.get_parent_ids()
-        self.parent_invs = []
-        for revision in self.parents:
+        self.parent_invs = [self.basis_inv]
+        for revision in self.parents[1:]:
             if self.branch.repository.has_revision(revision):
                 mutter('commit parent revision {%s}', revision)
                 inventory = self.branch.repository.get_inventory(revision)
@@ -550,6 +595,7 @@ class Commit(object):
         # in bugs like #46635.  Any reason not to use/enhance Tree.changes_from?
         # ADHB 11-07-2006
         mutter("Selecting files for commit with filter %s", self.specific_files)
+        assert self.work_inv.root is not None
         entries = self.work_inv.iter_entries()
         if not self.builder.record_root_entry:
             symbol_versioning.warn('CommitBuilders should support recording'
@@ -561,6 +607,38 @@ class Commit(object):
         for path, new_ie in entries:
             self._emit_progress_update()
             file_id = new_ie.file_id
+            try:
+                kind = self.work_tree.kind(file_id)
+                if kind == 'tree-reference' and self.recursive == 'down':
+                    # nested tree: commit in it
+                    sub_tree = WorkingTree.open(self.work_tree.abspath(path))
+                    # FIXME: be more comprehensive here:
+                    # this works when both trees are in --trees repository,
+                    # but when both are bound to a different repository,
+                    # it fails; a better way of approaching this is to 
+                    # finally implement the explicit-caches approach design
+                    # a while back - RBC 20070306.
+                    if (sub_tree.branch.repository.bzrdir.root_transport.base
+                        ==
+                        self.work_tree.branch.repository.bzrdir.root_transport.base):
+                        sub_tree.branch.repository = \
+                            self.work_tree.branch.repository
+                    try:
+                        sub_tree.commit(message=None, revprops=self.revprops,
+                            recursive=self.recursive,
+                            message_callback=self.message_callback,
+                            timestamp=self.timestamp, timezone=self.timezone,
+                            committer=self.committer,
+                            allow_pointless=self.allow_pointless,
+                            strict=self.strict, verbose=self.verbose,
+                            local=self.local, reporter=self.reporter)
+                    except errors.PointlessCommit:
+                        pass
+                if kind != new_ie.kind:
+                    new_ie = inventory.make_entry(kind, new_ie.name,
+                                                  new_ie.parent_id, file_id)
+            except errors.NoSuchFile:
+                pass
             # mutter('check %s {%s}', path, file_id)
             if (not self.specific_files or 
                 is_inside_or_parent_of_any(self.specific_files, path)):
@@ -574,7 +652,6 @@ class Commit(object):
                 else:
                     # this entry is new and not being committed
                     continue
-
             self.builder.record_entry_contents(ie, self.parent_invs, 
                 path, self.work_tree)
             # describe the nature of the change that has occurred relative to
