@@ -1,5 +1,6 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005, 2007 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
+#            and others
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -61,19 +62,28 @@ h=help
 up=pull
 """
 
+import os
+import sys
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 import errno
 from fnmatch import fnmatch
-import os
 import re
-import sys
 from StringIO import StringIO
 
 import bzrlib
-from bzrlib import errors, urlutils
-from bzrlib.osutils import pathjoin
-from bzrlib.trace import mutter, warning
+from bzrlib import (
+    errors,
+    osutils,
+    symbol_versioning,
+    urlutils,
+    win32utils,
+    )
 import bzrlib.util.configobj.configobj as configobj
+""")
+
+from bzrlib.trace import mutter, warning
 
 
 CHECK_IF_POSSIBLE=0
@@ -86,6 +96,30 @@ SIGN_ALWAYS=1
 SIGN_NEVER=2
 
 
+POLICY_NONE = 0
+POLICY_NORECURSE = 1
+POLICY_APPENDPATH = 2
+
+_policy_name = {
+    POLICY_NONE: None,
+    POLICY_NORECURSE: 'norecurse',
+    POLICY_APPENDPATH: 'appendpath',
+    }
+_policy_value = {
+    None: POLICY_NONE,
+    'none': POLICY_NONE,
+    'norecurse': POLICY_NORECURSE,
+    'appendpath': POLICY_APPENDPATH,
+    }
+
+
+STORE_LOCATION = POLICY_NONE
+STORE_LOCATION_NORECURSE = POLICY_NORECURSE
+STORE_LOCATION_APPENDPATH = POLICY_APPENDPATH
+STORE_BRANCH = 3
+STORE_GLOBAL = 4
+
+
 class ConfigObj(configobj.ConfigObj):
 
     def get_bool(self, section, key):
@@ -255,10 +289,26 @@ class IniBasedConfig(Config):
             raise errors.ParseConfigError(e.errors, e.config.filename)
         return self._parser
 
+    def _get_matching_sections(self):
+        """Return an ordered list of (section_name, extra_path) pairs.
+
+        If the section contains inherited configuration, extra_path is
+        a string containing the additional path components.
+        """
+        section = self._get_section()
+        if section is not None:
+            return [(section, '')]
+        else:
+            return []
+
     def _get_section(self):
         """Override this to define the section used by the config."""
         return "DEFAULT"
 
+    def _get_option_policy(self, section, option_name):
+        """Return the policy for the given (section, option_name) pair."""
+        return POLICY_NONE
+
     def _get_signature_checking(self):
         """See Config._get_signature_checking."""
         policy = self._get_user_option('check_signatures')
@@ -277,11 +327,28 @@ class IniBasedConfig(Config):
 
     def _get_user_option(self, option_name):
         """See Config._get_user_option."""
-        try:
-            return self._get_parser().get_value(self._get_section(),
-                                                option_name)
-        except KeyError:
-            pass
+        for (section, extra_path) in self._get_matching_sections():
+            try:
+                value = self._get_parser().get_value(section, option_name)
+            except KeyError:
+                continue
+            policy = self._get_option_policy(section, option_name)
+            if policy == POLICY_NONE:
+                return value
+            elif policy == POLICY_NORECURSE:
+                # norecurse items only apply to the exact path
+                if extra_path:
+                    continue
+                else:
+                    return value
+            elif policy == POLICY_APPENDPATH:
+                if extra_path:
+                    value = urlutils.join(value, extra_path)
+                return value
+            else:
+                raise AssertionError('Unexpected config policy %r' % policy)
+        else:
+            return None
 
     def _gpg_signing_command(self):
         """See Config.gpg_signing_command."""
@@ -379,13 +446,8 @@ class LocationConfig(IniBasedConfig):
             location = urlutils.local_path_from_url(location)
         self.location = location
 
-    def _get_section(self):
-        """Get the section we should look in for config items.
-
-        Returns None if none exists. 
-        TODO: perhaps return a NullSection that thunks through to the 
-              global config.
-        """
+    def _get_matching_sections(self):
+        """Return an ordered list of section names matching this location."""
         sections = self._get_parser()
         location_names = self.location.split('/')
         if self.location.endswith('/'):
@@ -415,21 +477,73 @@ class LocationConfig(IniBasedConfig):
             # if section is longer, no match.
             if len(section_names) > len(location_names):
                 continue
-            # if path is longer, and recurse is not true, no match
-            if len(section_names) < len(location_names):
-                try:
-                    if not self._get_parser()[section].as_bool('recurse'):
-                        continue
-                except KeyError:
-                    pass
-            matches.append((len(section_names), section))
-        if not len(matches):
-            return None
+            matches.append((len(section_names), section,
+                            '/'.join(location_names[len(section_names):])))
         matches.sort(reverse=True)
-        return matches[0][1]
+        sections = []
+        for (length, section, extra_path) in matches:
+            sections.append((section, extra_path))
+            # should we stop looking for parent configs here?
+            try:
+                if self._get_parser()[section].as_bool('ignore_parents'):
+                    break
+            except KeyError:
+                pass
+        return sections
 
-    def set_user_option(self, option, value):
+    def _get_option_policy(self, section, option_name):
+        """Return the policy for the given (section, option_name) pair."""
+        # check for the old 'recurse=False' flag
+        try:
+            recurse = self._get_parser()[section].as_bool('recurse')
+        except KeyError:
+            recurse = True
+        if not recurse:
+            return POLICY_NORECURSE
+
+        policy_key = option_name + ':policy'
+        try:
+            policy_name = self._get_parser()[section][policy_key]
+        except KeyError:
+            policy_name = None
+
+        return _policy_value[policy_name]
+
+    def _set_option_policy(self, section, option_name, option_policy):
+        """Set the policy for the given option name in the given section."""
+        # The old recurse=False option affects all options in the
+        # section.  To handle multiple policies in the section, we
+        # need to convert it to a policy_norecurse key.
+        try:
+            recurse = self._get_parser()[section].as_bool('recurse')
+        except KeyError:
+            pass
+        else:
+            symbol_versioning.warn(
+                'The recurse option is deprecated as of 0.14.  '
+                'The section "%s" has been converted to use policies.'
+                % section,
+                DeprecationWarning)
+            del self._get_parser()[section]['recurse']
+            if not recurse:
+                for key in self._get_parser()[section].keys():
+                    if not key.endswith(':policy'):
+                        self._get_parser()[section][key +
+                                                    ':policy'] = 'norecurse'
+
+        policy_key = option_name + ':policy'
+        policy_name = _policy_name[option_policy]
+        if policy_name is not None:
+            self._get_parser()[section][policy_key] = policy_name
+        else:
+            if policy_key in self._get_parser()[section]:
+                del self._get_parser()[section][policy_key]
+
+    def set_user_option(self, option, value, store=STORE_LOCATION):
         """Save option and its value in the configuration."""
+        assert store in [STORE_LOCATION,
+                         STORE_LOCATION_NORECURSE,
+                         STORE_LOCATION_APPENDPATH], 'bad storage policy'
         # FIXME: RBC 20051029 This should refresh the parser and also take a
         # file lock on locations.conf.
         conf_dir = os.path.dirname(self._get_filename())
@@ -443,6 +557,8 @@ class LocationConfig(IniBasedConfig):
         elif location + '/' in self._get_parser():
             location = location + '/'
         self._get_parser()[location][option]=value
+        # the allowed values of store match the config policies
+        self._set_option_policy(location, option, store)
         self._get_parser().write(file(self._get_filename(), 'wb'))
 
 
@@ -523,12 +639,13 @@ class BranchConfig(Config):
                 return value
         return None
 
-    def set_user_option(self, name, value, local=False):
-        if local is True:
-            self._get_location_config().set_user_option(name, value)
-        else:
+    def set_user_option(self, name, value, store=STORE_BRANCH):
+        if store == STORE_BRANCH:
             self._get_branch_data_config().set_option(value, name)
-
+        elif store == STORE_GLOBAL:
+            self._get_global_config().set_user_option(name, value)
+        else:
+            self._get_location_config().set_user_option(name, value, store)
 
     def _gpg_signing_command(self):
         """See Config.gpg_signing_command."""
@@ -552,7 +669,7 @@ class BranchConfig(Config):
         value = self._get_explicit_nickname()
         if value is not None:
             return value
-        return self.branch.base.split('/')[-2]
+        return urlutils.unescape(self.branch.base.split('/')[-2])
 
     def has_explicit_nickname(self):
         """Return true if a nickname has been explicitly assigned."""
@@ -594,37 +711,37 @@ def config_dir():
     base = os.environ.get('BZR_HOME', None)
     if sys.platform == 'win32':
         if base is None:
-            base = os.environ.get('APPDATA', None)
+            base = win32utils.get_appdata_location_unicode()
         if base is None:
             base = os.environ.get('HOME', None)
         if base is None:
             raise errors.BzrError('You must have one of BZR_HOME, APPDATA, or HOME set')
-        return pathjoin(base, 'bazaar', '2.0')
+        return osutils.pathjoin(base, 'bazaar', '2.0')
     else:
         # cygwin, linux, and darwin all have a $HOME directory
         if base is None:
             base = os.path.expanduser("~")
-        return pathjoin(base, ".bazaar")
+        return osutils.pathjoin(base, ".bazaar")
 
 
 def config_filename():
     """Return per-user configuration ini file filename."""
-    return pathjoin(config_dir(), 'bazaar.conf')
+    return osutils.pathjoin(config_dir(), 'bazaar.conf')
 
 
 def branches_config_filename():
     """Return per-user configuration ini file filename."""
-    return pathjoin(config_dir(), 'branches.conf')
+    return osutils.pathjoin(config_dir(), 'branches.conf')
 
 
 def locations_config_filename():
     """Return per-user configuration ini file filename."""
-    return pathjoin(config_dir(), 'locations.conf')
+    return osutils.pathjoin(config_dir(), 'locations.conf')
 
 
 def user_ignore_config_filename():
     """Return the user default ignore filename"""
-    return pathjoin(config_dir(), 'ignore')
+    return osutils.pathjoin(config_dir(), 'ignore')
 
 
 def _auto_user_id():
@@ -640,7 +757,16 @@ def _auto_user_id():
     """
     import socket
 
-    # XXX: Any good way to get real user name on win32?
+    if sys.platform == 'win32':
+        name = win32utils.get_user_name_unicode()
+        if name is None:
+            raise errors.BzrError("Cannot autodetect user name.\n"
+                                  "Please, set your name with command like:\n"
+                                  'bzr whoami "Your Name <name@domain.com>"')
+        host = win32utils.get_host_name_unicode()
+        if host is None:
+            host = socket.gethostname()
+        return name, (name + '@' + host)
 
     try:
         import pwd
@@ -698,8 +824,7 @@ def extract_email_address(e):
     """
     m = re.search(r'[\w+.-]+@[\w+.-]+', e)
     if not m:
-        raise errors.BzrError("%r doesn't seem to contain "
-                              "a reasonable email address" % e)
+        raise errors.NoEmailInUsername(e)
     return m.group(0)
 
 
