@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,21 +27,29 @@
 # created, but it's not for now.
 ROOT_ID = "TREE_ROOT"
 
-
-import collections
-import os.path
+import os
 import re
 import sys
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import collections
 import tarfile
-import types
-from warnings import warn
 
 import bzrlib
-from bzrlib import errors, osutils
-from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
-                            pathjoin, sha_strings)
-from bzrlib.errors import (NotVersionedError, InvalidEntryName,
-                           BzrError, BzrCheckError, BinaryFile)
+from bzrlib import (
+    errors,
+    generate_ids,
+    osutils,
+    symbol_versioning,
+    workingtree,
+    )
+""")
+
+from bzrlib.errors import (
+    BzrCheckError,
+    BzrError,
+    )
 from bzrlib.trace import mutter
 
 
@@ -82,17 +90,13 @@ class InventoryEntry(object):
     InventoryDirectory('123', 'src', parent_id='TREE_ROOT', revision=None)
     >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
     InventoryFile('2323', 'hello.c', parent_id='123', sha1=None, len=None)
-    >>> shouldbe = {0: '', 1: 'src', 2: pathjoin('src','hello.c')}
+    >>> shouldbe = {0: '', 1: 'src', 2: 'src/hello.c'}
     >>> for ix, j in enumerate(i.iter_entries()):
     ...   print (j[0] == shouldbe[ix], j[1])
     ... 
-    (True, InventoryDirectory('TREE_ROOT', '', parent_id=None, revision=None))
+    (True, InventoryDirectory('TREE_ROOT', u'', parent_id=None, revision=None))
     (True, InventoryDirectory('123', 'src', parent_id='TREE_ROOT', revision=None))
     (True, InventoryFile('2323', 'hello.c', parent_id='123', sha1=None, len=None))
-    >>> i.add(InventoryFile('2323', 'bye.c', '123'))
-    Traceback (most recent call last):
-    ...
-    BzrError: inventory already contains entry with id {2323}
     >>> i.add(InventoryFile('2324', 'bye.c', '123'))
     InventoryFile('2324', 'bye.c', parent_id='123', sha1=None, len=None)
     >>> i.add(InventoryDirectory('2325', 'wibble', '123'))
@@ -189,6 +193,9 @@ class InventoryEntry(object):
             if self.file_id in inv:
                 ie = inv[self.file_id]
                 assert ie.file_id == self.file_id
+                if ie.kind != self.kind:
+                    # Can't be a candidate if the kind has changed.
+                    continue
                 if ie.revision in candidates:
                     # same revision value in two different inventories:
                     # correct possible inconsistencies:
@@ -246,7 +253,7 @@ class InventoryEntry(object):
 
     def get_tar_item(self, root, dp, now, tree):
         """Get a tarfile item and a file stream for its content."""
-        item = tarfile.TarInfo(pathjoin(root, dp).encode('utf8'))
+        item = tarfile.TarInfo(osutils.pathjoin(root, dp).encode('utf8'))
         # TODO: would be cool to actually set it to the timestamp of the
         # revision it was last changed
         item.mtime = now
@@ -281,16 +288,19 @@ class InventoryEntry(object):
         """
         assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
-            raise InvalidEntryName(name=name)
+            raise errors.InvalidEntryName(name=name)
         self.executable = False
         self.revision = None
         self.text_sha1 = None
         self.text_size = None
         self.file_id = file_id
+        assert isinstance(file_id, (str, None.__class__)), \
+            'bad type %r for %r' % (type(file_id), file_id)
         self.name = name
         self.text_id = text_id
         self.parent_id = parent_id
         self.symlink_target = None
+        self.reference_revision = None
 
     def kind_character(self):
         """Return a short kind indicator useful for appending to names."""
@@ -311,7 +321,7 @@ class InventoryEntry(object):
         
         This is a template method - implement _put_on_disk in subclasses.
         """
-        fullpath = pathjoin(dest, dp)
+        fullpath = osutils.pathjoin(dest, dp)
         self._put_on_disk(fullpath, tree)
         # mutter("  export {%s} kind %s to %s", self.file_id,
         #         self.kind, fullpath)
@@ -325,7 +335,7 @@ class InventoryEntry(object):
 
     @staticmethod
     def versionable_kind(kind):
-        return (kind in ('file', 'directory', 'symlink'))
+        return (kind in ('file', 'directory', 'symlink', 'tree-reference'))
 
     def check(self, checker, rev_id, inv, tree):
         """Check this inventory entry is intact.
@@ -376,6 +386,8 @@ class InventoryEntry(object):
             return 'added'
         elif new_entry is None:
             return 'removed'
+        if old_entry.kind != new_entry.kind:
+            return 'modified'
         text_modified, meta_modified = new_entry.detect_changes(old_entry)
         if text_modified or meta_modified:
             modified = True
@@ -461,6 +473,7 @@ class InventoryEntry(object):
                 and (self.kind == other.kind)
                 and (self.revision == other.revision)
                 and (self.executable == other.executable)
+                and (self.reference_revision == other.reference_revision)
                 )
 
     def __ne__(self, other):
@@ -481,6 +494,8 @@ class InventoryEntry(object):
         # renamed
         elif previous_ie.name != self.name:
             compatible = False
+        elif previous_ie.kind != self.kind:
+            compatible = False
         return compatible
 
     def _read_tree_state(self, path, work_tree):
@@ -501,8 +516,8 @@ class InventoryEntry(object):
 class RootEntry(InventoryEntry):
 
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable', 
-                 'revision', 'symlink_target']
+                 'text_id', 'parent_id', 'children', 'executable',
+                 'revision', 'symlink_target', 'reference_revision']
 
     def _check(self, checker, rev_id, tree):
         """See InventoryEntry._check"""
@@ -514,9 +529,9 @@ class RootEntry(InventoryEntry):
         self.parent_id = None
         self.name = u''
         self.revision = None
-        warn('RootEntry is deprecated as of bzr 0.10.  Please use '
-             'InventoryDirectory instead.',
-            DeprecationWarning, stacklevel=2)
+        symbol_versioning.warn('RootEntry is deprecated as of bzr 0.10.'
+                               '  Please use InventoryDirectory instead.',
+                               DeprecationWarning, stacklevel=2)
 
     def __eq__(self, other):
         if not isinstance(other, RootEntry):
@@ -530,8 +545,8 @@ class InventoryDirectory(InventoryEntry):
     """A directory in an inventory."""
 
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable', 
-                 'revision', 'symlink_target']
+                 'text_id', 'parent_id', 'children', 'executable',
+                 'revision', 'symlink_target', 'reference_revision']
 
     def _check(self, checker, rev_id, tree):
         """See InventoryEntry._check"""
@@ -577,8 +592,8 @@ class InventoryFile(InventoryEntry):
     """A file in an inventory."""
 
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable', 
-                 'revision', 'symlink_target']
+                 'text_id', 'parent_id', 'children', 'executable',
+                 'revision', 'symlink_target', 'reference_revision']
 
     def _check(self, checker, tree_revision_id, tree):
         """See InventoryEntry._check"""
@@ -645,7 +660,7 @@ class InventoryFile(InventoryEntry):
             else:
                 text_diff(to_label, to_text,
                           from_label, from_text, output_to)
-        except BinaryFile:
+        except errors.BinaryFile:
             if reverse:
                 label_pair = (to_label, from_label)
             else:
@@ -677,7 +692,7 @@ class InventoryFile(InventoryEntry):
 
     def _put_on_disk(self, fullpath, tree):
         """See InventoryEntry._put_on_disk."""
-        pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
+        osutils.pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
         if tree.is_executable(self.file_id):
             os.chmod(fullpath, 0755)
 
@@ -725,8 +740,8 @@ class InventoryLink(InventoryEntry):
     """A file in an inventory."""
 
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable', 
-                 'revision', 'symlink_target']
+                 'text_id', 'parent_id', 'children', 'executable',
+                 'revision', 'symlink_target', 'reference_revision']
 
     def _check(self, checker, rev_id, tree):
         """See InventoryEntry._check"""
@@ -813,6 +828,33 @@ class InventoryLink(InventoryEntry):
             self.file_id, file_parents, self.symlink_target)
 
 
+class TreeReference(InventoryEntry):
+    
+    kind = 'tree-reference'
+    
+    def __init__(self, file_id, name, parent_id, revision=None,
+                 reference_revision=None):
+        InventoryEntry.__init__(self, file_id, name, parent_id)
+        self.revision = revision
+        self.reference_revision = reference_revision
+
+    def copy(self):
+        return TreeReference(self.file_id, self.name, self.parent_id,
+                             self.revision, self.reference_revision)
+
+    def _snapshot_text(self, file_parents, work_tree, commit_builder):
+        commit_builder.modified_reference(self.file_id, file_parents)
+
+    def _read_tree_state(self, path, work_tree):
+        """Populate fields in the inventory entry from the given tree.
+        """
+        self.reference_revision = work_tree.get_reference_revision(
+            self.file_id, path)
+
+    def _forget_tree_state(self):
+        self.reference_revision = None 
+
+
 class Inventory(object):
     """Inventory of versioned files in a tree.
 
@@ -849,6 +891,9 @@ class Inventory(object):
     ['', u'hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
     >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    Traceback (most recent call last):
+    BzrError: parent_id {TREE_ROOT} not in inventory
+    >>> inv.add(InventoryFile('123-123', 'hello.c', 'TREE_ROOT-12345678-12345678'))
     InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678', sha1=None, len=None)
     """
     def __init__(self, root_id=ROOT_ID, revision_id=None):
@@ -861,17 +906,12 @@ class Inventory(object):
         The inventory is created with a default root directory, with
         an id of None.
         """
-        # We are letting Branch.create() create a unique inventory
-        # root id. Rather than generating a random one here.
-        #if root_id is None:
-        #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')
         if root_id is not None:
-            self._set_root(InventoryDirectory(root_id, '', None))
+            assert root_id.__class__ == str
+            self._set_root(InventoryDirectory(root_id, u'', None))
         else:
             self.root = None
             self._byid = {}
-        # FIXME: this isn't ever used, changing it to self.revision may break
-        # things. TODO make everything use self.revision_id
         self.revision_id = revision_id
 
     def _set_root(self, ie):
@@ -898,7 +938,8 @@ class Inventory(object):
     def iter_entries(self, from_dir=None):
         """Return (path, entry) pairs, in order by name."""
         if from_dir is None:
-            assert self.root
+            if self.root is None:
+                return
             from_dir = self.root
             yield '', self.root
         elif isinstance(from_dir, basestring):
@@ -938,7 +979,7 @@ class Inventory(object):
                 # if we finished all children, pop it off the stack
                 stack.pop()
 
-    def iter_entries_by_dir(self, from_dir=None):
+    def iter_entries_by_dir(self, from_dir=None, specific_file_ids=None):
         """Iterate over the entries in a directory first order.
 
         This returns all entries for a directory before returning
@@ -948,14 +989,45 @@ class Inventory(object):
 
         :return: This yields (path, entry) pairs
         """
+        if specific_file_ids:
+            safe = osutils.safe_file_id
+            specific_file_ids = set(safe(fid) for fid in specific_file_ids)
         # TODO? Perhaps this should return the from_dir so that the root is
         # yielded? or maybe an option?
         if from_dir is None:
-            assert self.root
+            if self.root is None:
+                return
+            # Optimize a common case
+            if specific_file_ids is not None and len(specific_file_ids) == 1:
+                file_id = list(specific_file_ids)[0]
+                if file_id in self:
+                    yield self.id2path(file_id), self[file_id]
+                return 
             from_dir = self.root
-            yield '', self.root
+            if (specific_file_ids is None or 
+                self.root.file_id in specific_file_ids):
+                yield u'', self.root
         elif isinstance(from_dir, basestring):
             from_dir = self._byid[from_dir]
+
+        if specific_file_ids is not None:
+            # TODO: jam 20070302 This could really be done as a loop rather
+            #       than a bunch of recursive calls.
+            parents = set()
+            byid = self._byid
+            def add_ancestors(file_id):
+                if file_id not in byid:
+                    return
+                parent_id = byid[file_id].parent_id
+                if parent_id is None:
+                    return
+                if parent_id not in parents:
+                    parents.add(parent_id)
+                    add_ancestors(parent_id)
+            for file_id in specific_file_ids:
+                add_ancestors(file_id)
+        else:
+            parents = None
             
         stack = [(u'', from_dir)]
         while stack:
@@ -966,10 +1038,13 @@ class Inventory(object):
 
                 child_relpath = cur_relpath + child_name
 
-                yield child_relpath, child_ie
+                if (specific_file_ids is None or 
+                    child_ie.file_id in specific_file_ids):
+                    yield child_relpath, child_ie
 
                 if child_ie.kind == 'directory':
-                    child_dirs.append((child_relpath+'/', child_ie))
+                    if parents is None or child_ie.file_id in parents:
+                        child_dirs.append((child_relpath+'/', child_ie))
             stack.extend(reversed(child_dirs))
 
     def entries(self):
@@ -982,7 +1057,7 @@ class Inventory(object):
             kids = dir_ie.children.items()
             kids.sort()
             for name, ie in kids:
-                child_path = pathjoin(dir_path, name)
+                child_path = osutils.pathjoin(dir_path, name)
                 accum.append((child_path, ie))
                 if ie.kind == 'directory':
                     descend(ie, child_path)
@@ -1001,7 +1076,7 @@ class Inventory(object):
             kids.sort()
 
             for name, child_ie in kids:
-                child_path = pathjoin(parent_path, name)
+                child_path = osutils.pathjoin(parent_path, name)
                 descend(child_ie, child_path)
         descend(self.root, u'')
         return accum
@@ -1017,6 +1092,7 @@ class Inventory(object):
         >>> '456' in inv
         False
         """
+        file_id = osutils.safe_file_id(file_id)
         return (file_id in self._byid)
 
     def __getitem__(self, file_id):
@@ -1028,20 +1104,31 @@ class Inventory(object):
         >>> inv['123123'].name
         'hello.c'
         """
+        file_id = osutils.safe_file_id(file_id)
         try:
             return self._byid[file_id]
         except KeyError:
-            if file_id is None:
-                raise BzrError("can't look up file_id None")
-            else:
-                raise BzrError("file_id {%s} not in inventory" % file_id)
+            # really we're passing an inventory, not a tree...
+            raise errors.NoSuchId(self, file_id)
 
     def get_file_kind(self, file_id):
+        file_id = osutils.safe_file_id(file_id)
         return self._byid[file_id].kind
 
     def get_child(self, parent_id, filename):
+        parent_id = osutils.safe_file_id(parent_id)
         return self[parent_id].children.get(filename)
 
+    def _add_child(self, entry):
+        """Add an entry to the inventory, without adding it to its parent"""
+        if entry.file_id in self._byid:
+            raise BzrError("inventory already contains entry with id {%s}" %
+                           entry.file_id)
+        self._byid[entry.file_id] = entry
+        for child in getattr(entry, 'children', {}).itervalues():
+            self._add_child(child)
+        return entry
+
     def add(self, entry):
         """Add entry to inventory.
 
@@ -1051,28 +1138,25 @@ class Inventory(object):
         Returns the new entry object.
         """
         if entry.file_id in self._byid:
-            raise BzrError("inventory already contains entry with id {%s}" % entry.file_id)
+            raise errors.DuplicateFileId(entry.file_id,
+                                         self._byid[entry.file_id])
 
         if entry.parent_id is None:
             assert self.root is None and len(self._byid) == 0
-            self._set_root(entry)
-            return entry
-        if entry.parent_id == ROOT_ID:
-            assert self.root is not None, self
-            entry.parent_id = self.root.file_id
+            self.root = entry
+        else:
+            try:
+                parent = self._byid[entry.parent_id]
+            except KeyError:
+                raise BzrError("parent_id {%s} not in inventory" %
+                               entry.parent_id)
 
-        try:
-            parent = self._byid[entry.parent_id]
-        except KeyError:
-            raise BzrError("parent_id {%s} not in inventory" % entry.parent_id)
-
-        if entry.name in parent.children:
-            raise BzrError("%s is already versioned" %
-                    pathjoin(self.id2path(parent.file_id), entry.name))
-
-        self._byid[entry.file_id] = entry
-        parent.children[entry.name] = entry
-        return entry
+            if entry.name in parent.children:
+                raise BzrError("%s is already versioned" %
+                        osutils.pathjoin(self.id2path(parent.file_id),
+                        entry.name))
+            parent.children[entry.name] = entry
+        return self._add_child(entry)
 
     def add_path(self, relpath, kind, file_id=None, parent_id=None):
         """Add entry from a path.
@@ -1085,15 +1169,17 @@ class Inventory(object):
 
         if len(parts) == 0:
             if file_id is None:
-                file_id = bzrlib.workingtree.gen_root_id()
+                file_id = generate_ids.gen_root_id()
+            else:
+                file_id = osutils.safe_file_id(file_id)
             self.root = InventoryDirectory(file_id, '', None)
             self._byid = {self.root.file_id: self.root}
-            return
+            return self.root
         else:
             parent_path = parts[:-1]
             parent_id = self.path2id(parent_path)
             if parent_id is None:
-                raise NotVersionedError(path=parent_path)
+                raise errors.NotVersionedError(path=parent_path)
         ie = make_entry(kind, parts[-1], parent_id, file_id)
         return self.add(ie)
 
@@ -1109,6 +1195,7 @@ class Inventory(object):
         >>> '123' in inv
         False
         """
+        file_id = osutils.safe_file_id(file_id)
         ie = self[file_id]
 
         assert ie.parent_id is None or \
@@ -1147,11 +1234,12 @@ class Inventory(object):
 
     def _iter_file_id_parents(self, file_id):
         """Yield the parents of file_id up to the root."""
+        file_id = osutils.safe_file_id(file_id)
         while file_id is not None:
             try:
                 ie = self._byid[file_id]
             except KeyError:
-                raise BzrError("file_id {%s} not found in inventory" % file_id)
+                raise errors.NoSuchId(tree=None, file_id=file_id)
             yield ie
             file_id = ie.parent_id
 
@@ -1163,6 +1251,7 @@ class Inventory(object):
         is equal to the depth of the file in the tree, counting the
         root directory as depth 1.
         """
+        file_id = osutils.safe_file_id(file_id)
         p = []
         for parent in self._iter_file_id_parents(file_id):
             p.insert(0, parent.file_id)
@@ -1177,6 +1266,7 @@ class Inventory(object):
         >>> print i.id2path('foo-id')
         src/foo.c
         """
+        file_id = osutils.safe_file_id(file_id)
         # get all names, skipping root
         return '/'.join(reversed(
             [parent.name for parent in 
@@ -1193,15 +1283,20 @@ class Inventory(object):
 
         Returns None IFF the path is not found.
         """
-        if isinstance(name, types.StringTypes):
-            name = splitpath(name)
+        if isinstance(name, basestring):
+            name = osutils.splitpath(name)
 
         # mutter("lookup path %r" % name)
 
         parent = self.root
+        if parent is None:
+            return None
         for f in name:
             try:
-                cie = parent.children[f]
+                children = getattr(parent, 'children', None)
+                if children is None:
+                    return None
+                cie = children[f]
                 assert cie.name == f
                 assert cie.parent_id == parent.file_id
                 parent = cie
@@ -1215,6 +1310,7 @@ class Inventory(object):
         return bool(self.path2id(names))
 
     def has_id(self, file_id):
+        file_id = osutils.safe_file_id(file_id)
         return (file_id in self._byid)
 
     def remove_recursive_id(self, file_id):
@@ -1222,6 +1318,7 @@ class Inventory(object):
         
         :param file_id: A file_id to remove.
         """
+        file_id = osutils.safe_file_id(file_id)
         to_find_delete = [self._byid[file_id]]
         to_delete = []
         while to_find_delete:
@@ -1232,15 +1329,17 @@ class Inventory(object):
         for file_id in reversed(to_delete):
             ie = self[file_id]
             del self._byid[file_id]
-            if ie.parent_id is not None:
-                del self[ie.parent_id].children[ie.name]
+        if ie.parent_id is not None:
+            del self[ie.parent_id].children[ie.name]
 
     def rename(self, file_id, new_parent_id, new_name):
         """Move a file within the inventory.
 
         This can change either the name, or the parent, or both.
 
-        This does not move the working file."""
+        This does not move the working file.
+        """
+        file_id = osutils.safe_file_id(file_id)
         if not is_valid_name(new_name):
             raise BzrError("not an acceptable filename: %r" % new_name)
 
@@ -1264,6 +1363,17 @@ class Inventory(object):
         file_ie.name = new_name
         file_ie.parent_id = new_parent_id
 
+    def is_root(self, file_id):
+        file_id = osutils.safe_file_id(file_id)
+        return self.root is not None and file_id == self.root.file_id
+
+
+entry_factory = {
+    'directory': InventoryDirectory,
+    'file': InventoryFile,
+    'symlink': InventoryLink,
+    'tree-reference': TreeReference
+}
 
 def make_entry(kind, name, parent_id, file_id=None):
     """Create an inventory entry.
@@ -1274,8 +1384,14 @@ def make_entry(kind, name, parent_id, file_id=None):
     :param file_id: the file_id to use. if None, one will be created.
     """
     if file_id is None:
-        file_id = bzrlib.workingtree.gen_file_id(name)
+        file_id = generate_ids.gen_file_id(name)
+    else:
+        file_id = osutils.safe_file_id(file_id)
 
+    #------- This has been copied to bzrlib.dirstate.DirState.add, please
+    # keep them synchronised.
+    # we dont import normalized_filename directly because we want to be
+    # able to change the implementation at runtime for tests.
     norm_name, can_access = osutils.normalized_filename(name)
     if norm_name != name:
         if can_access:
@@ -1285,14 +1401,11 @@ def make_entry(kind, name, parent_id, file_id=None):
             #       if the error was raised with the full path
             raise errors.InvalidNormalization(name)
 
-    if kind == 'directory':
-        return InventoryDirectory(file_id, name, parent_id)
-    elif kind == 'file':
-        return InventoryFile(file_id, name, parent_id)
-    elif kind == 'symlink':
-        return InventoryLink(file_id, name, parent_id)
-    else:
+    try:
+        factory = entry_factory[kind]
+    except KeyError:
         raise BzrError("unknown kind %r" % kind)
+    return factory(file_id, name, parent_id)
 
 
 _NAME_RE = None
