@@ -220,6 +220,8 @@ class LockDir(object):
             self.transport.rename(tmpname, self._held_dir)
             self._lock_held = True
             self.confirm()
+        except errors.PermissionDenied:
+            raise
         except (PathError, DirectoryNotEmpty, FileExists, ResourceBusy), e:
             mutter("contention on %r: %s", self, e)
             raise LockContention(self)
