@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+# Copyright (C) 2004, 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,12 +17,21 @@
 # TODO: For things like --diff-prefix, we want a way to customize the display
 # of the option argument.
 
-import optparse
 import re
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import optparse
+
+from bzrlib import (
+    errors,
+    log,
+    registry,
+    revisionspec,
+    symbol_versioning,
+    )
+""")
 from bzrlib.trace import warning
-from bzrlib.revisionspec import RevisionSpec
-from bzrlib.errors import BzrCommandError
 
 
 def _parse_revision_str(revstr):
@@ -80,9 +89,10 @@ def _parse_revision_str(revstr):
     """
     # TODO: Maybe move this into revisionspec.py
     revs = []
+    # split on the first .. that is not followed by a / ?
     sep = re.compile("\\.\\.(?!/)")
     for x in sep.split(revstr):
-        revs.append(RevisionSpec.from_string(x or None))
+        revs.append(revisionspec.RevisionSpec.from_string(x or None))
     return revs
 
 
@@ -100,16 +110,22 @@ def get_merge_type(typestring):
         type_list = '\n'.join(lines)
         msg = "No known merge type %s. Supported types are:\n%s" %\
             (typestring, type_list)
-        raise BzrCommandError(msg)
+        raise errors.BzrCommandError(msg)
+
 
 class Option(object):
-    """Description of a command line option"""
+    """Description of a command line option
+    
+    :ivar _short_name: If this option has a single-letter name, this is it.
+    Otherwise None.
+    """
+
     # TODO: Some way to show in help a description of the option argument
 
     OPTIONS = {}
-    SHORT_OPTIONS = {}
 
-    def __init__(self, name, help='', type=None, argname=None):
+    def __init__(self, name, help='', type=None, argname=None,
+                 short_name=None):
         """Make a new command option.
 
         name -- regular name of the command, used in the double-dash
@@ -123,11 +139,10 @@ class Option(object):
 
         argname -- name of option argument, if any
         """
-        # TODO: perhaps a subclass that automatically does 
-        # --option, --no-option for reversible booleans
         self.name = name
         self.help = help
         self.type = type
+        self._short_name = short_name
         if type is None:
             assert argname is None
         elif argname is None:
@@ -135,13 +150,18 @@ class Option(object):
         self.argname = argname
 
     def short_name(self):
-        """Return the single character option for this command, if any.
+        if self._short_name:
+            return self._short_name
+        else:
+            # remove this when SHORT_OPTIONS is removed
+            # XXX: This is accessing a DeprecatedDict, so we call the super 
+            # method to avoid warnings
+            for (k, v) in dict.iteritems(Option.SHORT_OPTIONS):
+                if v == self:
+                    return k
 
-        Short options are globally registered.
-        """
-        for short, option in Option.SHORT_OPTIONS.iteritems():
-            if option is self:
-                return short
+    def set_short_name(self, short_name):
+        self._short_name = short_name
 
     def get_negation_name(self):
         if self.name.startswith('no-'):
@@ -185,13 +205,111 @@ class Option(object):
         yield self.name, self.short_name(), argname, self.help
 
 
+class RegistryOption(Option):
+    """Option based on a registry
+
+    The values for the options correspond to entries in the registry.  Input
+    must be a registry key.  After validation, it is converted into an object
+    using Registry.get or a caller-provided converter.
+    """
+
+    def validate_value(self, value):
+        """Validate a value name"""
+        if value not in self.registry:
+            raise errors.BadOptionValue(self.name, value)
+
+    def convert(self, value):
+        """Convert a value name into an output type"""
+        self.validate_value(value)
+        if self.converter is None:
+            return self.registry.get(value)
+        else:
+            return self.converter(value)
+
+    def __init__(self, name, help, registry, converter=None,
+        value_switches=False, title=None, enum_switch=True):
+        """
+        Constructor.
+
+        :param name: The option name.
+        :param help: Help for the option.
+        :param registry: A Registry containing the values
+        :param converter: Callable to invoke with the value name to produce
+            the value.  If not supplied, self.registry.get is used.
+        :param value_switches: If true, each possible value is assigned its
+            own switch.  For example, instead of '--format knit',
+            '--knit' can be used interchangeably.
+        :param enum_switch: If true, a switch is provided with the option name,
+            which takes a value.
+        """
+        Option.__init__(self, name, help, type=self.convert)
+        self.registry = registry
+        self.name = name
+        self.converter = converter
+        self.value_switches = value_switches
+        self.enum_switch = enum_switch
+        self.title = title
+        if self.title is None:
+            self.title = name
+
+    @staticmethod
+    def from_kwargs(name_, help=None, title=None, value_switches=False,
+                    enum_switch=True, **kwargs):
+        """Convenience method to generate string-map registry options
+
+        name, help, value_switches and enum_switch are passed to the
+        RegistryOption constructor.  Any other keyword arguments are treated
+        as values for the option, and they value is treated as the help.
+        """
+        reg = registry.Registry()
+        for name, help in kwargs.iteritems():
+            name = name.replace('_', '-')
+            reg.register(name, name, help=help)
+        return RegistryOption(name_, help, reg, title=title,
+            value_switches=value_switches, enum_switch=enum_switch)
+
+    def add_option(self, parser, short_name):
+        """Add this option to an Optparse parser"""
+        if self.value_switches:
+            parser = parser.add_option_group(self.title)
+        if self.enum_switch:
+            Option.add_option(self, parser, short_name)
+        if self.value_switches:
+            for key in self.registry.keys():
+                option_strings = ['--%s' % key]
+                if getattr(self.registry.get_info(key), 'hidden', False):
+                    help = optparse.SUPPRESS_HELP
+                else:
+                    help = self.registry.get_help(key)
+                parser.add_option(action='callback',
+                              callback=self._optparse_value_callback(key),
+                                  help=help,
+                                  *option_strings)
+
+    def _optparse_value_callback(self, cb_value):
+        def cb(option, opt, value, parser):
+            setattr(parser.values, self.name, self.type(cb_value))
+        return cb
+
+    def iter_switches(self):
+        """Iterate through the list of switches provided by the option
+
+        :return: an iterator of (name, short_name, argname, help)
+        """
+        for value in Option.iter_switches(self):
+            yield value
+        if self.value_switches:
+            for key in sorted(self.registry.keys()):
+                yield key, None, None, self.registry.get_help(key)
+
+
 class OptionParser(optparse.OptionParser):
     """OptionParser that raises exceptions instead of exiting"""
 
     DEFAULT_VALUE = object()
 
     def error(self, message):
-        raise BzrCommandError(message)
+        raise errors.BzrCommandError(message)
 
 
 def get_optparser(options):
@@ -199,10 +317,8 @@ def get_optparser(options):
 
     parser = OptionParser()
     parser.remove_option('--help')
-    short_options = dict((k.name, v) for v, k in 
-                         Option.SHORT_OPTIONS.iteritems())
     for option in options.itervalues():
-        option.add_option(parser, short_options.get(option.name))
+        option.add_option(parser, option.short_name())
     return parser
 
 
@@ -210,6 +326,24 @@ def _global_option(name, **kwargs):
     """Register o as a global option."""
     Option.OPTIONS[name] = Option(name, **kwargs)
 
+
+def _global_registry_option(name, help, registry, **kwargs):
+    Option.OPTIONS[name] = RegistryOption(name, help, registry, **kwargs)
+
+
+class MergeTypeRegistry(registry.Registry):
+
+    pass
+
+
+_merge_type_registry = MergeTypeRegistry()
+_merge_type_registry.register_lazy('merge3', 'bzrlib.merge', 'Merge3Merger',
+                                   "Native diff3-style merge")
+_merge_type_registry.register_lazy('diff3', 'bzrlib.merge', 'Diff3Merger',
+                                   "Merge using external diff3")
+_merge_type_registry.register_lazy('weave', 'bzrlib.merge', 'WeaveMerger',
+                                   "Weave-based merge")
+
 _global_option('all')
 _global_option('overwrite', help='Ignore differences between branches and '
                'overwrite unconditionally')
@@ -217,19 +351,21 @@ _global_option('basis', type=str)
 _global_option('bound')
 _global_option('diff-options', type=str)
 _global_option('help',
-               help='show help message')
-_global_option('file', type=unicode)
+               help='show help message',
+               short_name='h')
+_global_option('file', type=unicode, short_name='F')
 _global_option('force')
 _global_option('format', type=unicode)
 _global_option('forward')
-_global_option('message', type=unicode)
+_global_option('message', type=unicode,
+               short_name='m')
 _global_option('no-recurse')
-_global_option('prefix', type=str, 
-               help='Set prefixes to added to old and new filenames, as '
-                    'two values separated by a colon.')
 _global_option('profile',
                help='show performance profiling information')
-_global_option('revision', type=_parse_revision_str)
+_global_option('revision',
+               type=_parse_revision_str,
+               short_name='r',
+               help='See \'help revisionspec\' for details')
 _global_option('show-ids', 
                help='show internal object ids')
 _global_option('timezone', 
@@ -237,38 +373,47 @@ _global_option('timezone',
                help='display timezone as local, original, or utc')
 _global_option('unbound')
 _global_option('verbose',
-               help='display more information')
+               help='display more information',
+               short_name='v')
 _global_option('version')
 _global_option('email')
 _global_option('update')
-_global_option('log-format', type=str, help="Use this log format")
-_global_option('long', help='Use detailed log format. Same as --log-format long')
+_global_registry_option('log-format', "Use this log format",
+                        log.log_formatter_registry, value_switches=True,
+                        title='Log format')
+_global_option('long', help='Use detailed log format. Same as --log-format long',
+               short_name='l')
 _global_option('short', help='Use moderately short log format. Same as --log-format short')
 _global_option('line', help='Use log format with one line per revision. Same as --log-format line')
 _global_option('root', type=str)
 _global_option('no-backup')
-_global_option('merge-type', type=_parse_merge_type, 
-               help='Select a particular merge algorithm')
+_global_registry_option('merge-type', 'Select a particular merge algorithm',
+                        _merge_type_registry, value_switches=True,
+                        title='Merge algorithm')
 _global_option('pattern', type=str)
-_global_option('quiet')
+_global_option('quiet', short_name='q')
 _global_option('remember', help='Remember the specified location as a'
                ' default.')
 _global_option('reprocess', help='Reprocess to reduce spurious conflicts')
 _global_option('kind', type=str)
 _global_option('dry-run',
                help="show what would be done, but don't actually do anything")
+_global_option('name-from-revision', help='The path name in the old tree.')
 
 
-def _global_short(short_name, long_name):
-    assert short_name not in Option.SHORT_OPTIONS
-    Option.SHORT_OPTIONS[short_name] = Option.OPTIONS[long_name]
-    
-
-Option.SHORT_OPTIONS['F'] = Option.OPTIONS['file']
-Option.SHORT_OPTIONS['h'] = Option.OPTIONS['help']
-Option.SHORT_OPTIONS['m'] = Option.OPTIONS['message']
-Option.SHORT_OPTIONS['r'] = Option.OPTIONS['revision']
-Option.SHORT_OPTIONS['v'] = Option.OPTIONS['verbose']
-Option.SHORT_OPTIONS['l'] = Option.OPTIONS['long']
-Option.SHORT_OPTIONS['q'] = Option.OPTIONS['quiet']
-Option.SHORT_OPTIONS['p'] = Option.OPTIONS['prefix']
+# prior to 0.14 these were always globally registered; the old dict is
+# available for plugins that use it but it should not be used.
+Option.SHORT_OPTIONS = symbol_versioning.DeprecatedDict(
+    symbol_versioning.zero_fourteen,
+    'SHORT_OPTIONS',
+    {
+        'F': Option.OPTIONS['file'],
+        'h': Option.OPTIONS['help'],
+        'm': Option.OPTIONS['message'],
+        'r': Option.OPTIONS['revision'],
+        'v': Option.OPTIONS['verbose'],
+        'l': Option.OPTIONS['long'],
+        'q': Option.OPTIONS['quiet'],
+    },
+    'Set the short option name when constructing the Option.',
+    )
