@@ -1,8 +1,9 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,8 +21,7 @@ parallel knit.
 """
 
 
-import bzrlib
-import bzrlib.errors as errors
+from bzrlib import errors, osutils
 from bzrlib.knit import KnitVersionedFile, KnitPlainFactory
 from bzrlib.store.revision import RevisionStore
 from bzrlib.store.versioned import VersionedFileStore
@@ -65,12 +65,13 @@ class KnitRevisionStore(RevisionStore):
         self.get_revision_file(transaction).add_lines_with_ghosts(
             revision.revision_id,
             revision.parent_ids,
-            bzrlib.osutils.split_lines(revision_as_file.read()))
+            osutils.split_lines(revision_as_file.read()))
 
     def add_revision_signature_text(self, revision_id, signature_text, transaction):
         """See RevisionStore.add_revision_signature_text()."""
+        revision_id = osutils.safe_revision_id(revision_id)
         self.get_signature_file(transaction).add_lines(
-            revision_id, [], bzrlib.osutils.split_lines(signature_text))
+            revision_id, [], osutils.split_lines(signature_text))
 
     def all_revision_ids(self, transaction):
         """See RevisionStore.all_revision_ids()."""
@@ -79,6 +80,7 @@ class KnitRevisionStore(RevisionStore):
 
     def get_revisions(self, revision_ids, transaction):
         """See RevisionStore.get_revisions()."""
+        revision_ids = [osutils.safe_revision_id(r) for r in revision_ids]
         texts = self._get_serialized_revisions(revision_ids, transaction)
         revisions = []
         try:
@@ -87,10 +89,9 @@ class KnitRevisionStore(RevisionStore):
                 assert r.revision_id == revision_id
                 revisions.append(r)
         except SyntaxError, e:
-            raise errors.BzrError('failed to unpack revision_xml',
-                                   [revision_id,
-                                   str(e)])
-        return revisions 
+            raise errors.BzrError('failed to unpack revision_xml for %s: %s' %
+                                   (revision_id, str(e)))
+        return revisions
 
     def _get_serialized_revisions(self, revision_ids, transaction):
         texts = []
@@ -123,9 +124,10 @@ class KnitRevisionStore(RevisionStore):
 
     def has_revision_id(self, revision_id, transaction):
         """True if the store contains revision_id."""
+        revision_id = osutils.safe_revision_id(revision_id)
         return (revision_id is None
                 or self.get_revision_file(transaction).has_version(revision_id))
-        
+
     def _has_signature(self, revision_id, transaction):
         """See RevisionStore._has_signature()."""
         return self.get_signature_file(transaction).has_version(revision_id)
