@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@ import doctest
 import errno
 import logging
 import os
+from pprint import pformat
 import re
 import shlex
 import stat
@@ -50,11 +51,12 @@ from bzrlib import (
     memorytree,
     osutils,
     progress,
+    ui,
     urlutils,
     )
 import bzrlib.branch
 import bzrlib.commands
-import bzrlib.bundle.serializer
+import bzrlib.timestamp
 import bzrlib.export
 import bzrlib.inventory
 import bzrlib.iterablefile
@@ -91,7 +93,7 @@ default_transport = LocalURLServer
 
 MODULES_TO_TEST = []
 MODULES_TO_DOCTEST = [
-                      bzrlib.bundle.serializer,
+                      bzrlib.timestamp,
                       bzrlib.errors,
                       bzrlib.export,
                       bzrlib.inventory,
@@ -102,6 +104,8 @@ MODULES_TO_DOCTEST = [
                       bzrlib.store,
                       ]
 
+NUMBERED_DIRS = False   # dirs kind for TestCaseInTempDir (numbered or named)
+
 
 def packages_to_test():
     """Return a list of packages to test.
@@ -116,6 +120,7 @@ def packages_to_test():
     import bzrlib.tests.interrepository_implementations
     import bzrlib.tests.interversionedfile_implementations
     import bzrlib.tests.intertree_implementations
+    import bzrlib.tests.per_lock
     import bzrlib.tests.repository_implementations
     import bzrlib.tests.revisionstore_implementations
     import bzrlib.tests.tree_implementations
@@ -128,6 +133,7 @@ def packages_to_test():
             bzrlib.tests.interrepository_implementations,
             bzrlib.tests.interversionedfile_implementations,
             bzrlib.tests.intertree_implementations,
+            bzrlib.tests.per_lock,
             bzrlib.tests.repository_implementations,
             bzrlib.tests.revisionstore_implementations,
             bzrlib.tests.tree_implementations,
@@ -170,7 +176,7 @@ class ExtendedTestResult(unittest._TextTestResult):
                 revision_id = ''
             bench_history.write("--date %s %s\n" % (time.time(), revision_id))
         self._bench_history = bench_history
-        self.ui = bzrlib.ui.ui_factory
+        self.ui = ui.ui_factory
         self.num_tests = num_tests
         self.error_count = 0
         self.failure_count = 0
@@ -208,6 +214,7 @@ class ExtendedTestResult(unittest._TextTestResult):
     def startTest(self, test):
         unittest.TestResult.startTest(self, test)
         self.report_test_start(test)
+        test.number = self.count
         self._recordTestStartTime()
 
     def _recordTestStartTime(self):
@@ -267,7 +274,10 @@ class ExtendedTestResult(unittest._TextTestResult):
     def printErrorList(self, flavour, errors):
         for test, err in errors:
             self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % (flavour, self.getDescription(test)))
+            self.stream.write("%s: " % flavour)
+            if NUMBERED_DIRS:
+                self.stream.write('#%d ' % test.number)
+            self.stream.writeln(self.getDescription(test))
             if getattr(test, '_get_log', None) is not None:
                 print >>self.stream
                 print >>self.stream, \
@@ -324,17 +334,24 @@ class TextTestResult(ExtendedTestResult):
                 + ' ' 
                 + self._shortened_test_description(test))
 
+    def _test_description(self, test):
+        if NUMBERED_DIRS:
+            return '#%d %s' % (self.count,
+                               self._shortened_test_description(test))
+        else:
+            return self._shortened_test_description(test)
+
     def report_error(self, test, err):
         self.error_count += 1
         self.pb.note('ERROR: %s\n    %s\n', 
-            self._shortened_test_description(test),
+            self._test_description(test),
             err[1],
             )
 
     def report_failure(self, test, err):
         self.failure_count += 1
         self.pb.note('FAIL: %s\n    %s\n', 
-            self._shortened_test_description(test),
+            self._test_description(test),
             err[1],
             )
 
@@ -380,19 +397,33 @@ class VerboseTestResult(ExtendedTestResult):
         name = self._shortened_test_description(test)
         # width needs space for 6 char status, plus 1 for slash, plus 2 10-char
         # numbers, plus a trailing blank
-        self.stream.write(self._ellipsize_to_right(name,
-                            osutils.terminal_width()-30))
+        # when NUMBERED_DIRS: plus 5 chars on test number, plus 1 char on space
+        if NUMBERED_DIRS:
+            self.stream.write('%5d ' % self.count)
+            self.stream.write(self._ellipsize_to_right(name,
+                                osutils.terminal_width()-36))
+        else:
+            self.stream.write(self._ellipsize_to_right(name,
+                                osutils.terminal_width()-30))
         self.stream.flush()
 
+    def _error_summary(self, err):
+        indent = ' ' * 4
+        if NUMBERED_DIRS:
+            indent += ' ' * 6
+        return '%s%s' % (indent, err[1])
+
     def report_error(self, test, err):
         self.error_count += 1
-        self.stream.writeln('ERROR %s\n    %s' 
-                % (self._testTimeString(), err[1]))
+        self.stream.writeln('ERROR %s\n%s'
+                % (self._testTimeString(),
+                   self._error_summary(err)))
 
     def report_failure(self, test, err):
         self.failure_count += 1
-        self.stream.writeln(' FAIL %s\n    %s'
-                % (self._testTimeString(), err[1]))
+        self.stream.writeln(' FAIL %s\n%s'
+                % (self._testTimeString(),
+                   self._error_summary(err)))
 
     def report_success(self, test):
         self.stream.writeln('   OK %s' % self._testTimeString())
@@ -402,8 +433,10 @@ class VerboseTestResult(ExtendedTestResult):
         self.stream.flush()
 
     def report_skip(self, test, skip_excinfo):
-        print >>self.stream, ' SKIP %s' % self._testTimeString()
-        print >>self.stream, '     %s' % skip_excinfo[1]
+        self.skip_count += 1
+        self.stream.writeln(' SKIP %s\n%s'
+                % (self._testTimeString(),
+                   self._error_summary(skip_excinfo)))
 
 
 class TextTestRunner(object):
@@ -456,6 +489,10 @@ class TextTestRunner(object):
             self.stream.writeln(")")
         else:
             self.stream.writeln("OK")
+        if result.skip_count > 0:
+            skipped = result.skip_count
+            self.stream.writeln('%d test%s skipped' %
+                                (skipped, skipped != 1 and "s" or ""))
         result.report_cleaning_up()
         # This is still a little bogus, 
         # but only a little. Folk not using our testrunner will
@@ -475,7 +512,15 @@ class TextTestRunner(object):
                 else:
                     test_root = test_root.encode(
                         sys.getfilesystemencoding())
-                osutils.rmtree(test_root)
+                try:
+                    osutils.rmtree(test_root)
+                except OSError, e:
+                    if sys.platform == 'win32' and e.errno == errno.EACCES:
+                        print >>sys.stderr, ('Permission denied: '
+                                             'unable to remove testing dir '
+                                             '%s' % os.path.basename(test_root))
+                    else:
+                        raise
         else:
             note("Failed tests working directories are in '%s'\n", test_root)
         TestCaseWithMemoryTransport.TEST_ROOT = None
@@ -530,6 +575,71 @@ class StringIOWrapper(object):
             return setattr(self._cstring, name, val)
 
 
+class TestUIFactory(ui.CLIUIFactory):
+    """A UI Factory for testing.
+
+    Hide the progress bar but emit note()s.
+    Redirect stdin.
+    Allows get_password to be tested without real tty attached.
+    """
+
+    def __init__(self,
+                 stdout=None,
+                 stderr=None,
+                 stdin=None):
+        super(TestUIFactory, self).__init__()
+        if stdin is not None:
+            # We use a StringIOWrapper to be able to test various
+            # encodings, but the user is still responsible to
+            # encode the string and to set the encoding attribute
+            # of StringIOWrapper.
+            self.stdin = StringIOWrapper(stdin)
+        if stdout is None:
+            self.stdout = sys.stdout
+        else:
+            self.stdout = stdout
+        if stderr is None:
+            self.stderr = sys.stderr
+        else:
+            self.stderr = stderr
+
+    def clear(self):
+        """See progress.ProgressBar.clear()."""
+
+    def clear_term(self):
+        """See progress.ProgressBar.clear_term()."""
+
+    def clear_term(self):
+        """See progress.ProgressBar.clear_term()."""
+
+    def finished(self):
+        """See progress.ProgressBar.finished()."""
+
+    def note(self, fmt_string, *args, **kwargs):
+        """See progress.ProgressBar.note()."""
+        self.stdout.write((fmt_string + "\n") % args)
+
+    def progress_bar(self):
+        return self
+
+    def nested_progress_bar(self):
+        return self
+
+    def update(self, message, count=None, total=None):
+        """See progress.ProgressBar.update()."""
+
+    def get_non_echoed_password(self, prompt):
+        """Get password from stdin without trying to handle the echo mode"""
+        if prompt:
+            self.stdout.write(prompt)
+        password = self.stdin.readline()
+        if not password:
+            raise EOFError
+        if password[-1] == '\n':
+            password = password[:-1]
+        return password
+
+
 class TestCase(unittest.TestCase):
     """Base class for bzr unit tests.
     
@@ -569,14 +679,20 @@ class TestCase(unittest.TestCase):
         self._startLogFile()
         self._benchcalls = []
         self._benchtime = None
+        # prevent hooks affecting tests
+        self._preserved_hooks = bzrlib.branch.Branch.hooks
+        self.addCleanup(self._restoreHooks)
+        # this list of hooks must be kept in sync with the defaults
+        # in branch.py
+        bzrlib.branch.Branch.hooks = bzrlib.branch.BranchHooks()
 
     def _silenceUI(self):
         """Turn off UI for duration of test"""
         # by default the UI is off; tests can turn it on if they want it.
-        saved = bzrlib.ui.ui_factory
+        saved = ui.ui_factory
         def _restore():
-            bzrlib.ui.ui_factory = saved
-        bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
+            ui.ui_factory = saved
+        ui.ui_factory = ui.SilentUIFactory()
         self.addCleanup(_restore)
 
     def _ndiff_strings(self, a, b):
@@ -594,6 +710,22 @@ class TestCase(unittest.TestCase):
                                   charjunk=lambda x: False)
         return ''.join(difflines)
 
+    def assertEqual(self, a, b, message=''):
+        try:
+            if a == b:
+                return
+        except UnicodeError, e:
+            # If we can't compare without getting a UnicodeError, then
+            # obviously they are different
+            mutter('UnicodeError: %s', e)
+        if message:
+            message += '\n'
+        raise AssertionError("%snot equal:\na = %s\nb = %s\n"
+            % (message,
+               pformat(a, indent=4), pformat(b, indent=4)))
+
+    assertEquals = assertEqual
+
     def assertEqualDiff(self, a, b, message=None):
         """Assert two texts are equal, if not raise an exception.
         
@@ -624,7 +756,7 @@ class TestCase(unittest.TestCase):
     def assertContainsRe(self, haystack, needle_re):
         """Assert that a contains something matching a regular expression."""
         if not re.search(needle_re, haystack):
-            raise AssertionError('pattern "%s" not found in "%s"'
+            raise AssertionError('pattern "%r" not found in "%r"'
                     % (needle_re, haystack))
 
     def assertNotContainsRe(self, haystack, needle_re):
@@ -661,9 +793,39 @@ class TestCase(unittest.TestCase):
                 excName = str(excClass)
             raise self.failureException, "%s not raised" % excName
 
-    def assertIs(self, left, right):
+    def assertRaises(self, excClass, func, *args, **kwargs):
+        """Assert that a callable raises a particular exception.
+
+        :param excClass: As for the except statement, this may be either an
+        exception class, or a tuple of classes.
+
+        Returns the exception so that you can examine it.
+        """
+        try:
+            func(*args, **kwargs)
+        except excClass, e:
+            return e
+        else:
+            if getattr(excClass,'__name__', None) is not None:
+                excName = excClass.__name__
+            else:
+                # probably a tuple
+                excName = str(excClass)
+            raise self.failureException, "%s not raised" % excName
+
+    def assertIs(self, left, right, message=None):
         if not (left is right):
-            raise AssertionError("%r is not %r." % (left, right))
+            if message is not None:
+                raise AssertionError(message)
+            else:
+                raise AssertionError("%r is not %r." % (left, right))
+
+    def assertIsNot(self, left, right, message=None):
+        if (left is right):
+            if message is not None:
+                raise AssertionError(message)
+            else:
+                raise AssertionError("%r is %r." % (left, right))
 
     def assertTransportMode(self, transport, path, mode):
         """Fail if a path does not have mode mode.
@@ -725,7 +887,7 @@ class TestCase(unittest.TestCase):
         expected_first_warning = symbol_versioning.deprecation_string(
             a_callable, deprecation_format)
         if len(call_warnings) == 0:
-            self.fail("No assertion generated by call to %s" %
+            self.fail("No deprecation warning generated by call to %s" %
                 a_callable)
         self.assertEqual(expected_first_warning, call_warnings[0])
         return result
@@ -792,7 +954,7 @@ class TestCase(unittest.TestCase):
         new_env = {
             'BZR_HOME': None, # Don't inherit BZR_HOME to all the tests.
             'HOME': os.getcwd(),
-            'APPDATA': os.getcwd(),
+            'APPDATA': None,  # bzr now use Win32 API and don't rely on APPDATA
             'BZR_EMAIL': None,
             'BZREMAIL': None, # may still be present in the environment
             'EMAIL': None,
@@ -825,6 +987,9 @@ class TestCase(unittest.TestCase):
         for name, value in self.__old_env.iteritems():
             osutils.set_or_unset_env(name, value)
 
+    def _restoreHooks(self):
+        bzrlib.branch.Branch.hooks = self._preserved_hooks
+
     def tearDown(self):
         self._runCleanups()
         unittest.TestCase.tearDown(self)
@@ -858,8 +1023,11 @@ class TestCase(unittest.TestCase):
         """
         # TODO: Perhaps this should keep running cleanups even if 
         # one of them fails?
-        for cleanup_fn in reversed(self._cleanups):
-            cleanup_fn()
+
+        # Actually pop the cleanups from the list so tearDown running
+        # twice is safe (this happens for skipped tests).
+        while self._cleanups:
+            self._cleanups.pop()()
 
     def log(self, *args):
         mutter(*args)
@@ -881,7 +1049,14 @@ class TestCase(unittest.TestCase):
                 logfile.close()
             if not keep_log_file:
                 self._log_contents = log_contents
-                os.remove(self._log_file_name)
+                try:
+                    os.remove(self._log_file_name)
+                except OSError, e:
+                    if sys.platform == 'win32' and e.errno == errno.EACCES:
+                        print >>sys.stderr, ('Unable to delete log file '
+                                             ' %r' % self._log_file_name)
+                    else:
+                        raise
             return log_contents
         else:
             return "DELETED log file to reduce memory footprint"
@@ -916,8 +1091,6 @@ class TestCase(unittest.TestCase):
         """
         if encoding is None:
             encoding = bzrlib.user_encoding
-        if stdin is not None:
-            stdin = StringIO(stdin)
         stdout = StringIOWrapper()
         stderr = StringIOWrapper()
         stdout.encoding = encoding
@@ -929,11 +1102,8 @@ class TestCase(unittest.TestCase):
         handler.setLevel(logging.INFO)
         logger = logging.getLogger('')
         logger.addHandler(handler)
-        old_ui_factory = bzrlib.ui.ui_factory
-        bzrlib.ui.ui_factory = bzrlib.tests.blackbox.TestUIFactory(
-            stdout=stdout,
-            stderr=stderr)
-        bzrlib.ui.ui_factory.stdin = stdin
+        old_ui_factory = ui.ui_factory
+        ui.ui_factory = TestUIFactory(stdin=stdin, stdout=stdout, stderr=stderr)
 
         cwd = None
         if working_dir is not None:
@@ -944,14 +1114,15 @@ class TestCase(unittest.TestCase):
             saved_debug_flags = frozenset(debug.debug_flags)
             debug.debug_flags.clear()
             try:
-                result = self.apply_redirected(stdin, stdout, stderr,
+                result = self.apply_redirected(ui.ui_factory.stdin,
+                                               stdout, stderr,
                                                bzrlib.commands.run_bzr_catch_errors,
                                                argv)
             finally:
                 debug.debug_flags.update(saved_debug_flags)
         finally:
             logger.removeHandler(handler)
-            bzrlib.ui.ui_factory = old_ui_factory
+            ui.ui_factory = old_ui_factory
             if cwd is not None:
                 os.chdir(cwd)
 
@@ -976,6 +1147,8 @@ class TestCase(unittest.TestCase):
         where it may be useful for debugging.  See also run_captured.
 
         :param stdin: A string to be used as stdin for the command.
+        :param retcode: The status code the command should return
+        :param working_dir: The directory to run the command in
         """
         retcode = kwargs.pop('retcode', 0)
         encoding = kwargs.pop('encoding', None)
@@ -1405,10 +1578,10 @@ class TestCaseWithMemoryTransport(TestCase):
                     t.mkdir('.')
                 except errors.FileExists:
                     pass
+            if format is None:
+                format = 'default'
             if isinstance(format, basestring):
                 format = bzrdir.format_registry.make_bzrdir(format)
-            elif format is None:
-                format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
             return format.initialize_on_transport(t)
         except errors.UninitializableFormat:
             raise TestSkipped("Format %s is not initializable." % format)
@@ -1425,7 +1598,7 @@ class TestCaseWithMemoryTransport(TestCase):
 
     def overrideEnvironmentForTesting(self):
         os.environ['HOME'] = self.test_home_dir
-        os.environ['APPDATA'] = self.test_home_dir
+        os.environ['BZR_HOME'] = self.test_home_dir
         
     def setUp(self):
         super(TestCaseWithMemoryTransport, self).setUp()
@@ -1469,6 +1642,23 @@ class TestCaseInTempDir(TestCaseWithMemoryTransport):
         For TestCaseInTempDir we create a temporary directory based on the test
         name and then create two subdirs - test and home under it.
         """
+        if NUMBERED_DIRS:       # strongly recommended on Windows
+                                # due the path length limitation (260 chars)
+            candidate_dir = '%s/%dK/%05d' % (self.TEST_ROOT,
+                                             int(self.number/1000),
+                                             self.number)
+            os.makedirs(candidate_dir)
+            self.test_home_dir = candidate_dir + '/home'
+            os.mkdir(self.test_home_dir)
+            self.test_dir = candidate_dir + '/work'
+            os.mkdir(self.test_dir)
+            os.chdir(self.test_dir)
+            # put name of test inside
+            f = file(candidate_dir + '/name', 'w')
+            f.write(self.id())
+            f.close()
+            return
+        # Else NAMED DIRS
         # shorten the name, to avoid test failures due to path length
         short_id = self.id().replace('bzrlib.tests.', '') \
                    .replace('__main__.', '')[-100:]
@@ -1535,15 +1725,20 @@ class TestCaseInTempDir(TestCaseWithMemoryTransport):
         """Fail if path does not contain 'content'."""
         self.failUnlessExists(path)
         # TODO: jam 20060427 Shouldn't this be 'rb'?
-        self.assertEqualDiff(content, open(path, 'r').read())
+        f = file(path, 'r')
+        try:
+            s = f.read()
+        finally:
+            f.close()
+        self.assertEqualDiff(content, s)
 
     def failUnlessExists(self, path):
         """Fail unless path, which may be abs or relative, exists."""
-        self.failUnless(osutils.lexists(path))
+        self.failUnless(osutils.lexists(path),path+" does not exist")
 
     def failIfExists(self, path):
         """Fail if path, which may be abs or relative, exists."""
-        self.failIf(osutils.lexists(path))
+        self.failIf(osutils.lexists(path),path+" exists")
 
 
 class TestCaseWithTransport(TestCaseInTempDir):
@@ -1626,6 +1821,15 @@ class TestCaseWithTransport(TestCaseInTempDir):
             self.fail("path %s is not a directory; has mode %#o"
                       % (relpath, mode))
 
+    def assertTreesEqual(self, left, right):
+        """Check that left and right have the same content and properties."""
+        # we use a tree delta to check for equality of the content, and we
+        # manually check for equality of other things such as the parents list.
+        self.assertEqual(left.get_parent_ids(), right.get_parent_ids())
+        differences = left.changes_from(right)
+        self.assertFalse(differences.has_changed(),
+            "Trees %r and %r are different: %r" % (left, right, differences))
+
     def setUp(self):
         super(TestCaseWithTransport, self).setUp()
         self.__server = None
@@ -1658,9 +1862,27 @@ def filter_suite_by_re(suite, pattern):
     return result
 
 
+def sort_suite_by_re(suite, pattern):
+    first = []
+    second = []
+    filter_re = re.compile(pattern)
+    for test in iter_suite_tests(suite):
+        if filter_re.search(test.id()):
+            first.append(test)
+        else:
+            second.append(test)
+    return TestUtil.TestSuite(first + second)
+
+
 def run_suite(suite, name='test', verbose=False, pattern=".*",
               stop_on_failure=False, keep_output=False,
-              transport=None, lsprof_timed=None, bench_history=None):
+              transport=None, lsprof_timed=None, bench_history=None,
+              matching_tests_first=None,
+              numbered_dirs=None):
+    global NUMBERED_DIRS
+    if numbered_dirs is not None:
+        NUMBERED_DIRS = bool(numbered_dirs)
+
     TestCase._gather_lsprof_in_benchmarks = lsprof_timed
     if verbose:
         verbosity = 2
@@ -1673,7 +1895,10 @@ def run_suite(suite, name='test', verbose=False, pattern=".*",
                             bench_history=bench_history)
     runner.stop_on_failure=stop_on_failure
     if pattern != '.*':
-        suite = filter_suite_by_re(suite, pattern)
+        if matching_tests_first:
+            suite = sort_suite_by_re(suite, pattern)
+        else:
+            suite = filter_suite_by_re(suite, pattern)
     result = runner.run(suite)
     return result.wasSuccessful()
 
@@ -1683,7 +1908,9 @@ def selftest(verbose=False, pattern=".*", stop_on_failure=True,
              transport=None,
              test_suite_factory=None,
              lsprof_timed=None,
-             bench_history=None):
+             bench_history=None,
+             matching_tests_first=None,
+             numbered_dirs=None):
     """Run the whole test suite under the enhanced runner"""
     # XXX: Very ugly way to do this...
     # Disable warning about old formats because we don't want it to disturb
@@ -1705,7 +1932,9 @@ def selftest(verbose=False, pattern=".*", stop_on_failure=True,
                      stop_on_failure=stop_on_failure, keep_output=keep_output,
                      transport=transport,
                      lsprof_timed=lsprof_timed,
-                     bench_history=bench_history)
+                     bench_history=bench_history,
+                     matching_tests_first=matching_tests_first,
+                     numbered_dirs=numbered_dirs)
     finally:
         default_transport = old_transport
 
@@ -1732,10 +1961,13 @@ def test_suite():
                    'bzrlib.tests.test_config',
                    'bzrlib.tests.test_conflicts',
                    'bzrlib.tests.test_decorators',
+                   'bzrlib.tests.test_delta',
                    'bzrlib.tests.test_diff',
+                   'bzrlib.tests.test_dirstate',
                    'bzrlib.tests.test_doc_generate',
                    'bzrlib.tests.test_errors',
                    'bzrlib.tests.test_escaped_store',
+                   'bzrlib.tests.test_extract',
                    'bzrlib.tests.test_fetch',
                    'bzrlib.tests.test_ftp_transport',
                    'bzrlib.tests.test_generate_docs',
@@ -1746,6 +1978,7 @@ def test_suite():
                    'bzrlib.tests.test_hashcache',
                    'bzrlib.tests.test_http',
                    'bzrlib.tests.test_http_response',
+                   'bzrlib.tests.test_https_ca_bundle',
                    'bzrlib.tests.test_identitymap',
                    'bzrlib.tests.test_ignores',
                    'bzrlib.tests.test_inv',
@@ -1759,6 +1992,7 @@ def test_suite():
                    'bzrlib.tests.test_merge',
                    'bzrlib.tests.test_merge3',
                    'bzrlib.tests.test_merge_core',
+                   'bzrlib.tests.test_merge_directive',
                    'bzrlib.tests.test_missing',
                    'bzrlib.tests.test_msgeditor',
                    'bzrlib.tests.test_nonascii',
@@ -1785,12 +2019,16 @@ def test_suite():
                    'bzrlib.tests.test_smart_add',
                    'bzrlib.tests.test_smart_transport',
                    'bzrlib.tests.test_source',
+                   'bzrlib.tests.test_ssh_transport',
                    'bzrlib.tests.test_status',
                    'bzrlib.tests.test_store',
+                   'bzrlib.tests.test_subsume',
                    'bzrlib.tests.test_symbol_versioning',
+                   'bzrlib.tests.test_tag',
                    'bzrlib.tests.test_testament',
                    'bzrlib.tests.test_textfile',
                    'bzrlib.tests.test_textmerge',
+                   'bzrlib.tests.test_timestamp',
                    'bzrlib.tests.test_trace',
                    'bzrlib.tests.test_transactions',
                    'bzrlib.tests.test_transform',
@@ -1808,6 +2046,7 @@ def test_suite():
                    'bzrlib.tests.test_weave',
                    'bzrlib.tests.test_whitebox',
                    'bzrlib.tests.test_workingtree',
+                   'bzrlib.tests.test_workingtree_4',
                    'bzrlib.tests.test_wsgi',
                    'bzrlib.tests.test_xml',
                    ]
@@ -1833,7 +2072,20 @@ def test_suite():
             raise
     for name, plugin in bzrlib.plugin.all_plugins().items():
         if getattr(plugin, 'test_suite', None) is not None:
-            suite.addTest(plugin.test_suite())
+            default_encoding = sys.getdefaultencoding()
+            try:
+                plugin_suite = plugin.test_suite()
+            except ImportError, e:
+                bzrlib.trace.warning(
+                    'Unable to test plugin "%s": %s', name, e)
+            else:
+                suite.addTest(plugin_suite)
+            if default_encoding != sys.getdefaultencoding():
+                bzrlib.trace.warning(
+                    'Plugin "%s" tried to reset default encoding to: %s', name,
+                    sys.getdefaultencoding())
+                reload(sys)
+                sys.setdefaultencoding(default_encoding)
     return suite
 
 
