@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,9 @@
 import os
 from cStringIO import StringIO
 
+from bzrlib import (
+    bzrdir,
+    )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import (BzrDir, BzrDirFormat, BzrDirMetaFormat1)
 from bzrlib.osutils import getcwd
@@ -99,14 +102,34 @@ class TestBoundBranches(TestCaseWithTransport):
 
         self.run_bzr('unbind', retcode=3)
 
+    def test_bind_branch6(self):
+        branch1 = self.make_branch('branch1', format='dirstate-tags')
+        os.chdir('branch1')
+        error = self.run_bzr('bind', retcode=3)[1]
+        self.assertContainsRe(error, 'no previous location known')
+
+    def setup_rebind(self, format):
+        branch1 = self.make_branch('branch1')
+        branch2 = self.make_branch('branch2', format=format)
+        branch2.bind(branch1)
+        branch2.unbind()
+
+    def test_rebind_branch6(self):
+        self.setup_rebind('dirstate-tags')
+        os.chdir('branch2')
+        self.run_bzr('bind')
+        b = Branch.open('.')
+        self.assertContainsRe(b.get_bound_location(), '\/branch1\/$')
+
+    def test_rebind_branch5(self):
+        self.setup_rebind('knit')
+        os.chdir('branch2')
+        error = self.run_bzr('bind', retcode=3)[1]
+        self.assertContainsRe(error, 'old locations')
+
     def init_meta_branch(self, path):
-        old_format = BzrDirFormat.get_default_format()
-        BzrDirFormat.set_default_format(BzrDirMetaFormat1())
-        try:
-            return BzrDir.create_branch_convenience(
-                path, BzrDirMetaFormat1())
-        finally:
-            BzrDirFormat.set_default_format(old_format)
+        format = bzrdir.format_registry.make_bzrdir('default')
+        return BzrDir.create_branch_convenience(path, format=format)
 
     def test_bound_commit(self):
         bzr = self.run_bzr
@@ -351,7 +374,8 @@ class TestBoundBranches(TestCaseWithTransport):
 
         bzr('cat-revision', new_rev_id)
 
-    def test_pull_overwrite_fails(self):
+    def test_pull_overwrite(self):
+        # XXX: This test should be moved to branch-implemenations/test_pull
         bzr = self.run_bzr
         self.create_branches()
 
@@ -375,17 +399,8 @@ class TestBoundBranches(TestCaseWithTransport):
         self.check_revno(2)
         self.check_revno(2, '../base')
 
-        # It might be possible that we want pull --overwrite to
-        # actually succeed.
-        # If we want it, just change this test to make sure that 
-        # both base and child are updated properly
-        bzr('pull', '--overwrite', '../other', retcode=3)
-
-        # It should fail without changing the local revision
-        self.check_revno(2)
-        self.check_revno(2, '../base')
-
-    # TODO: jam 20051230 Test that commit & pull fail when the branch we 
-    #       are bound to is not available
-
+        bzr('pull', '--overwrite', '../other')
 
+        # both the local and master should have been updated.
+        self.check_revno(4)
+        self.check_revno(4, '../base')
