@@ -1,4 +1,4 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,6 +16,9 @@
 
 import os
 
+from bzrlib import (
+    conflicts
+    )
 from bzrlib.workingtree import WorkingTree
 from bzrlib.tests.blackbox import ExternalBase
 
@@ -33,6 +36,7 @@ class TestConflicts(ExternalBase):
         self.runbzr('init')
         file('myfile', 'wb').write('contentsa\n')
         file('my_other_file', 'wb').write('contentsa\n')
+        os.mkdir('mydir')
         self.runbzr('add')
         self.runbzr('commit -m new')
         os.chdir('..')
@@ -40,23 +44,29 @@ class TestConflicts(ExternalBase):
         os.chdir('b')
         file('myfile', 'wb').write('contentsb\n')
         file('my_other_file', 'wb').write('contentsb\n')
+        self.runbzr('mv mydir mydir2')
         self.runbzr('commit -m change')
         os.chdir('../a')
         file('myfile', 'wb').write('contentsa2\n')
         file('my_other_file', 'wb').write('contentsa2\n')
+        self.runbzr('mv mydir mydir3')
         self.runbzr('commit -m change')
         self.runbzr('merge ../b', retcode=1)
 
     def test_conflicts(self):
         conflicts = self.runbzr('conflicts', backtick=True)
-        self.assertEqual(len(conflicts.splitlines()), 2)
+        self.assertEqual(3, len(conflicts.splitlines()))
+
+    def test_conflicts_text(self):
+        conflicts = self.run_bzr('conflicts', '--text')[0].splitlines()
+        self.assertEqual(['my_other_file', 'myfile'], conflicts)
 
     def test_resolve(self):
-        self.runbzr('resolve', retcode=3)
         self.runbzr('resolve myfile')
         conflicts = self.runbzr('conflicts', backtick=True)
-        self.assertEqual(len(conflicts.splitlines()), 1)
+        self.assertEqual(2, len(conflicts.splitlines()))
         self.runbzr('resolve my_other_file')
+        self.runbzr('resolve mydir2')
         conflicts = self.runbzr('conflicts', backtick=True)
         self.assertEqual(len(conflicts.splitlines()), 0)
 
@@ -80,3 +90,21 @@ class TestConflicts(ExternalBase):
                 self.fail("tree still contains conflicts: %r" % conflicts)
         finally:
             os.chdir(orig_dir)
+
+    def test_auto_resolve(self):
+        """Text conflicts can be resolved automatically"""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file',
+            '<<<<<<<\na\n=======\n>>>>>>>\n')])
+        tree.add('file', 'file_id')
+        self.assertEqual(tree.kind('file_id'), 'file')
+        file_conflict = conflicts.TextConflict('file', file_id='file_id')
+        tree.set_conflicts(conflicts.ConflictList([file_conflict]))
+        os.chdir('tree')
+        note = self.run_bzr('resolve', retcode=1)[1]
+        self.assertContainsRe(note, '0 conflict\\(s\\) auto-resolved.')
+        self.assertContainsRe(note,
+            'Remaining conflicts:\nText conflict in file')
+        self.build_tree_contents([('file', 'a\n')])
+        note = self.run_bzr('resolve')[1]
+        self.assertContainsRe(note, 'All conflicts resolved.')
