@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 by Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,7 +34,8 @@ def subst_dates(string):
                   'YYYY-MM-DD HH:MM:SS +ZZZZ', string)
 
 
-class TestDiff(ExternalBase):
+class DiffBase(ExternalBase):
+    """Base class with common setup method"""
 
     def make_example_branch(self):
         # FIXME: copied from test_too_much -- share elsewhere?
@@ -46,6 +47,9 @@ class TestDiff(ExternalBase):
         tree.add(['goodbye'])
         tree.commit('setup')
 
+
+class TestDiff(DiffBase):
+
     def test_diff(self):
         self.make_example_branch()
         file('hello', 'wt').write('hello world!')
@@ -75,6 +79,12 @@ class TestDiff(ExternalBase):
 
 ''')
 
+    def test_diff_illegal_prefix_value(self):
+        # There was an error in error reporting for this option
+        out, err = self.runbzr('diff --prefix old/', retcode=3)
+        self.assertContainsRe(err,
+            '--prefix expects two values separated by a colon')
+
     def test_diff_p1(self):
         """diff -p1 produces lkml-style diffs"""
         self.make_example_branch()
@@ -114,6 +124,10 @@ class TestDiff(ExternalBase):
         out, err = self.runbzr('diff does-not-exist', retcode=3)
         self.assertContainsRe(err, 'not versioned.*does-not-exist')
 
+    def test_diff_illegal_revision_specifiers(self):
+        out, err = self.runbzr('diff -r 1..23..123', retcode=3)
+        self.assertContainsRe(err, 'one or two revision specifiers')
+
     def test_diff_unversioned(self):
         # Get an error when diffing a non-versioned file.
         # (Malone #3619)
@@ -230,7 +244,7 @@ class TestCheckoutDiff(TestDiff):
         os.chdir('checkouts')
 
 
-class TestDiffLabels(TestDiff):
+class TestDiffLabels(DiffBase):
 
     def test_diff_label_removed(self):
         super(TestDiffLabels, self).make_example_branch()
@@ -258,7 +272,7 @@ class TestDiffLabels(TestDiff):
         self.assertTrue("=== renamed file 'hello' => 'gruezi'" in diff[0])
 
 
-class TestExternalDiff(TestDiff):
+class TestExternalDiff(DiffBase):
 
     def test_external_diff(self):
         """Test that we can spawn an external diff process"""
@@ -272,6 +286,7 @@ class TestExternalDiff(TestDiff):
             os.environ['BZR_PROGRESS_BAR'] = 'none'
             out, err = self.run_bzr_subprocess('diff', '-r', '1',
                                                '--diff-options', '-ub',
+                                               universal_newlines=True,
                                                retcode=None)
         finally:
             if orig_progress is None:
@@ -289,3 +304,13 @@ class TestExternalDiff(TestDiff):
                                    "+++ goodbye\t")
         self.assertEndsWith(out, "\n@@ -0,0 +1 @@\n"
                                  "+baz\n\n")
+
+
+class TestDiffOutput(DiffBase):
+
+    def test_diff_output(self):
+        # check that output doesn't mangle line-endings
+        self.make_example_branch()
+        file('hello', 'wb').write('hello world!\n')
+        output = self.run_bzr_subprocess('diff', retcode=1)[0]
+        self.assert_('\n+hello world!\n' in output)
