@@ -1,5 +1,4 @@
-# Copyright (C) 2006 by Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +20,10 @@
 import sys
 
 import bzrlib
-from bzrlib import repository
+from bzrlib import (
+    bzrdir,
+    repository,
+    )
 from bzrlib.osutils import format_date
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
@@ -42,10 +44,7 @@ class TestInfo(ExternalBase):
         transport = self.get_transport()
 
         # Create initial standalone branch
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirFormat6())
-        tree1 = self.make_branch_and_tree('standalone')
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        tree1 = self.make_branch_and_tree('standalone', 'weave')
         self.build_tree(['standalone/a'])
         tree1.add('a')
         branch1 = tree1.branch
@@ -134,8 +133,7 @@ Revision store:
         # Branch and bind to standalone, needs upgrade to metadir
         # (creates backup as unknown)
         branch1.bzrdir.sprout('bound')
-        knit1_format = bzrlib.bzrdir.BzrDirMetaFormat1()
-        knit1_format.repository_format = repository.RepositoryFormatKnit1()
+        knit1_format = bzrdir.format_registry.make_bzrdir('knit')
         bzrlib.upgrade.upgrade('bound', knit1_format)
         branch3 = bzrlib.bzrdir.BzrDir.open('bound').open_branch()
         branch3.bind(branch1)
@@ -152,7 +150,7 @@ Related branches:
 Format:
        control: Meta directory format 1
   working tree: %s
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -178,6 +176,7 @@ Revision store:
        branch1.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
        bound_tree._format.get_format_description(),      
+       branch3._format.get_format_description(),
        branch3.repository._format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -188,10 +187,8 @@ Revision store:
         self.assertEqual('', err)
 
         # Checkout standalone (same as above, but does not have parent set)
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(knit1_format)
-        branch4 = bzrlib.bzrdir.BzrDir.create_branch_convenience('checkout')
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        branch4 = bzrlib.bzrdir.BzrDir.create_branch_convenience('checkout',
+            format=knit1_format)
         branch4.bind(branch1)
         branch4.bzrdir.open_workingtree().update()
         out, err = self.runbzr('info checkout --verbose')
@@ -238,14 +235,7 @@ Revision store:
         self.assertEqual('', err)
 
         # Lightweight checkout (same as above, different branch and repository)
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
-        transport.mkdir('lightcheckout')
-        dir5 = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('lightcheckout')
-        bzrlib.branch.BranchReferenceFormat().initialize(dir5, branch1)
-        dir5.create_workingtree()
-        tree5 = dir5.open_workingtree()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+        tree5 = branch1.create_checkout('lightcheckout', lightweight=True)
         branch5 = tree5.branch
         out, err = self.runbzr('info lightcheckout')
         self.assertEqualDiff(
@@ -255,7 +245,7 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 3
+  working tree: Working tree format 4
         branch: Branch format 4
     repository: Weave repository format 6
 
@@ -436,7 +426,7 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 3
+  working tree: Working tree format 4
         branch: Branch format 4
     repository: Weave repository format 6
 
@@ -470,6 +460,7 @@ Revision store:
 
     def test_info_standalone_no_tree(self):
         # create standalone branch without a working tree
+        format = bzrdir.format_registry.make_bzrdir('default')
         branch = self.make_branch('branch')
         repo = branch.repository
         out, err = self.runbzr('info branch')
@@ -479,7 +470,7 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 Branch history:
@@ -489,17 +480,17 @@ Revision store:
          0 revisions
          0 KiB
 """ % (branch.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
     def test_info_shared_repository(self):
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        format = bzrdir.format_registry.make_bzrdir('knit')
         transport = self.get_transport()
 
         # Create shared repository
-        repo = self.make_repository('repo', shared=True)
+        repo = self.make_repository('repo', shared=True, format=format)
         repo.set_make_working_trees(False)
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
@@ -514,13 +505,14 @@ Revision store:
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
         # Create branch inside shared repository
         repo.bzrdir.root_transport.mkdir('branch')
-        branch1 = repo.bzrdir.create_branch_convenience('repo/branch')
+        branch1 = repo.bzrdir.create_branch_convenience('repo/branch',
+            format=format)
         out, err = self.runbzr('info repo/branch')
         self.assertEqualDiff(
 """Location:
@@ -529,7 +521,7 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 Branch history:
@@ -539,19 +531,19 @@ Revision store:
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
         # Create lightweight checkout
         transport.mkdir('tree')
         transport.mkdir('tree/lightcheckout')
-        dir2 = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('tree/lightcheckout')
-        bzrlib.branch.BranchReferenceFormat().initialize(dir2, branch1)
-        dir2.create_workingtree()
-        tree2 = dir2.open_workingtree()
+        tree2 = branch1.create_checkout('tree/lightcheckout', 
+            lightweight=True)
         branch2 = tree2.branch
-        self.assertCheckoutStatusOutput('tree/lightcheckout', tree2, shared_repo=repo)
+        self.assertCheckoutStatusOutput('tree/lightcheckout', tree2, 
+                   shared_repo=repo)
 
         # Create normal checkout
         tree3 = branch1.create_checkout('tree/checkout')
@@ -573,8 +565,8 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
+  working tree: Working tree format 4
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -599,7 +591,8 @@ Revision store:
          %d KiB
 """ % (tree2.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -616,8 +609,8 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
+  working tree: Working tree format 4
+        branch: %s
     repository: %s
 
 Branch is out of date: missing 1 revision.
@@ -640,7 +633,8 @@ Revision store:
          0 KiB
 """ % (tree3.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
@@ -656,8 +650,8 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
+  working tree: Working tree format 4
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -681,7 +675,8 @@ Revision store:
          1 revision
          %d KiB
 """ % (tree3.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -702,8 +697,8 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 3
-        branch: Branch format 5
+  working tree: Working tree format 4
+        branch: %s
     repository: %s
 
 Working tree is out of date: missing 1 revision.
@@ -730,7 +725,8 @@ Revision store:
          %d KiB
 """ % (tree2.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_last,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -747,7 +743,7 @@ Revision store:
 
 Format:
        control: Meta directory format 1
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 Branch history:
@@ -761,7 +757,8 @@ Revision store:
          2 revisions
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_last,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -783,22 +780,19 @@ Revision store:
          2 revisions
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
        repo._revision_store.total_size(repo.get_transaction())[1] / 1024,
        ), out)
         self.assertEqual('', err)
 
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
-
     def test_info_shared_repository_with_trees(self):
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        format = bzrdir.format_registry.make_bzrdir('knit')
         transport = self.get_transport()
 
         # Create shared repository with working trees
-        repo = self.make_repository('repo', shared=True)
+        repo = self.make_repository('repo', shared=True, format=format)
         repo.set_make_working_trees(True)
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
@@ -815,13 +809,14 @@ Revision store:
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
         # Create two branches
         repo.bzrdir.root_transport.mkdir('branch1')
-        branch1 = repo.bzrdir.create_branch_convenience('repo/branch1')
+        branch1 = repo.bzrdir.create_branch_convenience('repo/branch1',
+            format=format)
         branch2 = branch1.bzrdir.sprout('repo/branch2').open_branch()
 
         # Empty first branch
@@ -834,7 +829,7 @@ Revision store:
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -855,7 +850,8 @@ Revision store:
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
@@ -875,7 +871,7 @@ Revision store:
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -898,7 +894,8 @@ Revision store:
          1 revision
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -919,7 +916,7 @@ Related branches:
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -941,7 +938,8 @@ Revision store:
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
        repo._revision_store.total_size(repo.get_transaction())[1] / 1024,
@@ -963,7 +961,7 @@ Related branches:
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -987,7 +985,8 @@ Revision store:
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
@@ -1011,23 +1010,20 @@ Revision store:
          1 revision
          %d KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
        repo._revision_store.total_size(repo.get_transaction())[1] / 1024,
        ),
        out)
         self.assertEqual('', err)
-
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
     
     def test_info_shared_repository_with_tree_in_root(self):
-        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        format = bzrdir.format_registry.make_bzrdir('knit')
         transport = self.get_transport()
 
         # Create shared repository with working trees
-        repo = self.make_repository('repo', shared=True)
+        repo = self.make_repository('repo', shared=True, format=format)
         repo.set_make_working_trees(True)
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
@@ -1044,7 +1040,7 @@ Revision store:
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
@@ -1061,7 +1057,7 @@ Revision store:
 Format:
        control: Meta directory format 1
   working tree: Working tree format 3
-        branch: Branch format 5
+        branch: %s
     repository: %s
 
 In the working tree:
@@ -1081,12 +1077,11 @@ Revision store:
          0 revisions
          0 KiB
 """ % (repo.bzrdir.root_transport.base,
-       repo._format.get_format_description(),
+       format.get_branch_format().get_format_description(),
+       format.repository_format.get_format_description(),
        ), out)
         self.assertEqual('', err)
 
-        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
-
     def assertCheckoutStatusOutput(self, 
         command_string, lco_tree, shared_repo=None,
         repo_branch=None,
@@ -1162,7 +1157,7 @@ Revision store:
 Format:
        control: Meta directory format 1
   working tree: %s
-        branch: Branch format 5
+        branch: %s
     repository: %s
 %s
 In the working tree:
@@ -1184,6 +1179,7 @@ Revision store:
 """ %  (tree_data,
         branch_data,
         lco_tree._format.get_format_description(),
+        lco_tree.branch._format.get_format_description(),
         lco_tree.branch.repository._format.get_format_description(),
         expected_lock_output,
         verbose_info,
