@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,8 +23,8 @@ import os
 
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.conflicts import ConflictList
-from bzrlib.osutils import abspath
+from bzrlib.conflicts import ConflictList, ContentsConflict
+from bzrlib.osutils import abspath, file_kind
 from bzrlib.tests.blackbox import ExternalBase
 import bzrlib.urlutils as urlutils
 from bzrlib.workingtree import WorkingTree
@@ -169,7 +169,7 @@ class TestMerge(ExternalBase):
         
         base = urlutils.local_path_from_url(branch_a.base)
         self.assertEquals(out, 'Merging from remembered location %s\n' % (base,))
-        self.assertEquals(err, 'All changes applied successfully.\n')
+        self.assertEquals(err, '+N  b\nAll changes applied successfully.\n')
         self.assertEquals(abspath(branch_b.get_parent()), abspath(parent))
         # re-open tree as external runbzr modified it
         tree_b = branch_b.bzrdir.open_workingtree()
@@ -177,7 +177,7 @@ class TestMerge(ExternalBase):
         # test explicit --remember
         out, err = self.runbzr('merge ../branch_c --remember')
         self.assertEquals(out, '')
-        self.assertEquals(err, 'All changes applied successfully.\n')
+        self.assertEquals(err, '+N  c\nAll changes applied successfully.\n')
         self.assertEquals(abspath(branch_b.get_parent()),
                           abspath(branch_c.bzrdir.root_transport.base))
         # re-open tree as external runbzr modified it
@@ -232,10 +232,61 @@ class TestMerge(ExternalBase):
         tree_a.rename_one('file_1', 'file_i')
         tree_a.commit('commit 2')
         tree_a.rename_one('file_2', 'file_ii')
-        os.chdir('b')
-        self.run_bzr('merge', '../a', '--uncommitted')
-        self.failUnlessExists('file_1')
-        self.failUnlessExists('file_ii')
+        ## os.chdir('b')
+        self.run_bzr('merge', 'a', '--uncommitted', '-d', 'b')
+        self.failUnlessExists('b/file_1')
+        self.failUnlessExists('b/file_ii')
         tree_b.revert([])
-        self.run_bzr_error(('Cannot use --uncommitted and --revision',), 
-                           'merge', '../a', '--uncommitted', '-r1')
+        self.run_bzr_error(('Cannot use --uncommitted and --revision',),
+                           'merge', '/a', '--uncommitted', '-r1',
+                           '-d', 'b')
+
+    def pullable_branch(self):
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        file('goodbye', 'wt').write('quux')
+        self.runbzr(['commit', '-m', "mode u's are always good"])
+        os.chdir('../a')
+
+    def pullable_branch(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/file'])
+        tree_a.add(['file'])
+        self.id1 = tree_a.commit('commit 1')
+        
+        tree_b = self.make_branch_and_tree('b')
+        tree_b.pull(tree_a.branch)
+        file('b/file', 'wb').write('foo')
+        self.id2 = tree_b.commit('commit 2')
+
+    def test_merge_pull(self):
+        self.pullable_branch()
+        os.chdir('a')
+        (out, err) = self.run_bzr('merge', '--pull', '../b')
+        self.assertContainsRe(err, 'Now on revision 2\\.')
+        tree_a = WorkingTree.open('.')
+        self.assertEqual([self.id2], tree_a.get_parent_ids())
+
+    def test_merge_kind_change(self):
+        tree_a = self.make_branch_and_tree('tree_a')
+        self.build_tree_contents([('tree_a/file', 'content_1')])
+        tree_a.add('file', 'file-id')
+        tree_a.commit('added file')
+        tree_b = tree_a.bzrdir.sprout('tree_b').open_workingtree()
+        os.unlink('tree_a/file')
+        self.build_tree(['tree_a/file/'])
+        tree_a.commit('changed file to directory')
+        os.chdir('tree_b')
+        self.run_bzr('merge', '../tree_a')
+        self.assertEqual('directory', file_kind('file'))
+        tree_b.revert([])
+        self.assertEqual('file', file_kind('file'))
+        self.build_tree_contents([('file', 'content_2')])
+        tree_b.commit('content change')
+        self.run_bzr('merge', '../tree_a', retcode=1)
+        self.assertEqual(tree_b.conflicts(),
+                         [ContentsConflict('file', file_id='file-id')])
