@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -56,7 +56,9 @@ class ReSign(TestCaseInTempDir):
     def assertEqualSignature(self, repo, revision_id):
         """Assert a signature is stored correctly in repository."""
         self.assertEqual(
-            Testament.from_revision(repo, revision_id).as_short_text(),
+            '-----BEGIN PSEUDO-SIGNED CONTENT-----\n' +
+            Testament.from_revision(repo, revision_id).as_short_text() +
+            '-----END PSEUDO-SIGNED CONTENT-----\n',
             repo.get_signature_text(revision_id))
 
     def test_resign(self):
