@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 by Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,10 +18,9 @@
 
 import os
 
-from bzrlib.tests import TestCaseInTempDir
-import bzrlib.bzrdir
 from bzrlib.bzrdir import BzrDir
 import bzrlib.errors as errors
+from bzrlib.tests import TestCaseInTempDir
 
 class TestSharedRepo(TestCaseInTempDir):
 
@@ -29,7 +28,7 @@ class TestSharedRepo(TestCaseInTempDir):
         out, err = self.run_bzr("init-repository", "a")
         self.assertEqual(out, "")
         self.assertEqual(err, "")
-        dir = bzrlib.bzrdir.BzrDir.open('a')
+        dir = BzrDir.open('a')
         self.assertIs(dir.open_repository().is_shared(), True)
         self.assertRaises(errors.NotBranchError, dir.open_branch)
         self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)        
@@ -46,23 +45,23 @@ class TestSharedRepo(TestCaseInTempDir):
     def test_init(self):
         self.run_bzr("init-repo", "a")
         self.run_bzr("init", "--format=default", "a/b")
-        dir = bzrlib.bzrdir.BzrDir.open('a')
+        dir = BzrDir.open('a')
         self.assertIs(dir.open_repository().is_shared(), True)
         self.assertRaises(errors.NotBranchError, dir.open_branch)
         self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
-        bdir = bzrlib.bzrdir.BzrDir.open('a/b')
+        bdir = BzrDir.open('a/b')
         bdir.open_branch()
         self.assertRaises(errors.NoRepositoryPresent, bdir.open_repository)
-        self.assertRaises(errors.NoWorkingTree, bdir.open_workingtree)
+        wt = bdir.open_workingtree()
 
     def test_branch(self):
         self.run_bzr("init-repo", "a")
         self.run_bzr("init", "--format=default", "a/b")
         self.run_bzr('branch', 'a/b', 'a/c')
-        cdir = bzrlib.bzrdir.BzrDir.open('a/c')
+        cdir = BzrDir.open('a/c')
         cdir.open_branch()
         self.assertRaises(errors.NoRepositoryPresent, cdir.open_repository)
-        self.assertRaises(errors.NoWorkingTree, cdir.open_workingtree)
+        cdir.open_workingtree()
 
     def test_branch_tree(self):
         self.run_bzr("init-repo", "--trees", "a")
@@ -72,9 +71,27 @@ class TestSharedRepo(TestCaseInTempDir):
         self.run_bzr("commit", "-m", "bar", "b/hello")
 
         self.run_bzr('branch', 'b', 'a/c')
-        cdir = bzrlib.bzrdir.BzrDir.open('a/c')
+        cdir = BzrDir.open('a/c')
         cdir.open_branch()
         self.assertRaises(errors.NoRepositoryPresent, cdir.open_repository)
         self.failUnlessExists('a/c/hello')
         cdir.open_workingtree()
 
+    def test_trees_default(self):
+        # 0.15 switched to trees by default
+        self.run_bzr("init-repo", "repo")
+        repo = BzrDir.open("repo").open_repository()
+        self.assertEqual(True, repo.make_working_trees())
+
+    def test_trees_argument(self):
+        # Supplying the --trees argument should be harmless,
+        # as it was previously non-default we need to get it right.
+        self.run_bzr("init-repo", "--trees", "trees")
+        repo = BzrDir.open("trees").open_repository()
+        self.assertEqual(True, repo.make_working_trees())
+
+    def test_no_trees_argument(self):
+        # --no-trees should make it so that there is no working tree
+        self.run_bzr("init-repo", "--no-trees", "notrees")
+        repo = BzrDir.open("notrees").open_repository()
+        self.assertEqual(False, repo.make_working_trees())
