@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 by Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ interface later, they will be non blackbox tests.
 
 from cStringIO import StringIO
 import codecs
-from os import mkdir, chdir
+from os import mkdir, chdir, rmdir, unlink
 import sys
 from tempfile import TemporaryFile
 
@@ -40,21 +40,21 @@ from bzrlib.workingtree import WorkingTree
 
 class BranchStatus(TestCaseWithTransport):
     
-    def assertStatus(self, output_lines, working_tree,
-        revision=None):
+    def assertStatus(self, expected_lines, working_tree,
+        revision=None, short=False):
         """Run status in working_tree and look for output.
         
-        :param output_lines: The lines to look for.
+        :param expected_lines: The lines to look for.
         :param working_tree: The tree to run status in.
         """
-        output_string = self.status_string(working_tree, revision)
-        self.assertEqual(output_lines, output_string.splitlines(True))
+        output_string = self.status_string(working_tree, revision, short)
+        self.assertEqual(expected_lines, output_string.splitlines(True))
     
-    def status_string(self, wt, revision=None):
+    def status_string(self, wt, revision=None, short=False):
         # use a real file rather than StringIO because it doesn't handle
         # Unicode very well.
         tof = codecs.getwriter('utf-8')(TemporaryFile())
-        show_tree_status(wt, to_file=tof, revision=revision)
+        show_tree_status(wt, to_file=tof, revision=revision, short=short)
         tof.seek(0)
         return tof.read().decode('utf-8')
 
@@ -75,6 +75,11 @@ class BranchStatus(TestCaseWithTransport):
                 '  hello.c\n',
             ],
             wt)
+        self.assertStatus([
+                '?   bye.c\n',
+                '?   hello.c\n',
+            ],
+            wt, short=True)
 
         # add a commit to allow showing pending merges.
         wt.commit('create a parent to allow testing merge output')
@@ -88,6 +93,12 @@ class BranchStatus(TestCaseWithTransport):
                 '  pending@pending-0-0\n',
             ],
             wt)
+        self.assertStatus([
+                '?   bye.c\n',
+                '?   hello.c\n',
+                'P   pending@pending-0-0\n',
+            ],
+            wt, short=True)
 
     def test_branch_status_revisions(self):
         """Tests branch status with revisions"""
@@ -132,19 +143,35 @@ class BranchStatus(TestCaseWithTransport):
         wt.commit(u"\N{TIBETAN DIGIT TWO} Empty commit 2")
         merge(["./branch", -1], [None, None], this_dir = './copy')
         message = self.status_string(wt2)
-        self.assert_(message.startswith("pending merges:\n"))
-        self.assert_(message.endswith("Empty commit 2\n")) 
+        self.assertStartsWith(message, "pending merges:\n")
+        self.assertEndsWith(message, "Empty commit 2\n")
         wt2.commit("merged")
         # must be long to make sure we see elipsis at the end
-        wt.commit("Empty commit 3 " + 
-                   "blah blah blah blah " * 10)
+        wt.commit("Empty commit 3 " +
+                   "blah blah blah blah " * 100)
         merge(["./branch", -1], [None, None], this_dir = './copy')
         message = self.status_string(wt2)
-        self.assert_(message.startswith("pending merges:\n"))
+        self.assertStartsWith(message, "pending merges:\n")
         self.assert_("Empty commit 3" in message)
-        self.assert_(message.endswith("...\n")) 
+        self.assertEndsWith(message, "...\n")
 
-    def test_branch_status_specific_files(self): 
+    def test_tree_status_ignores(self):
+        """Tests branch status with ignores"""
+        wt = self.make_branch_and_tree('.')
+        self.run_bzr('ignore', '*~')
+        wt.commit('commit .bzrignore')
+        self.build_tree(['foo.c', 'foo.c~'])
+        self.assertStatus([
+                'unknown:\n',
+                '  foo.c\n',
+                ],
+                wt)
+        self.assertStatus([
+                '?   foo.c\n',
+                ],
+                wt, short=True)
+
+    def test_tree_status_specific_files(self):
         """Tests branch status with given specific files"""
         wt = self.make_branch_and_tree('.')
         b = wt.branch
@@ -154,16 +181,22 @@ class BranchStatus(TestCaseWithTransport):
         wt.add('test.c')
         wt.commit('testing')
         
-        tof = StringIO()
-        show_tree_status(wt, to_file=tof)
-        tof.seek(0)
-        self.assertEquals(tof.readlines(),
-                          ['unknown:\n',
-                           '  bye.c\n',
-                           '  dir2\n',
-                           '  directory/hello.c\n'
-                           ])
+        self.assertStatus([
+                'unknown:\n',
+                '  bye.c\n',
+                '  dir2/\n',
+                '  directory/hello.c\n'
+                ],
+                wt)
 
+        self.assertStatus([
+                '?   bye.c\n',
+                '?   dir2/\n',
+                '?   directory/hello.c\n'
+                ],
+                wt, short=True)
+
+        tof = StringIO()
         self.assertRaises(errors.PathsDoNotExist,
                           show_tree_status,
                           wt, specific_files=['bye.c','test.c','absent.c'], 
@@ -176,13 +209,23 @@ class BranchStatus(TestCaseWithTransport):
                           ['unknown:\n',
                            '  directory/hello.c\n'
                            ])
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['directory'], to_file=tof,
+                         short=True)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(), ['?   directory/hello.c\n'])
+
         tof = StringIO()
         show_tree_status(wt, specific_files=['dir2'], to_file=tof)
         tof.seek(0)
         self.assertEquals(tof.readlines(),
                           ['unknown:\n',
-                           '  dir2\n'
+                           '  dir2/\n'
                            ])
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['dir2'], to_file=tof, short=True)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(), ['?   dir2/\n'])
 
     def test_status_nonexistent_file(self):
         # files that don't exist in either the basis tree or working tree
@@ -191,6 +234,22 @@ class BranchStatus(TestCaseWithTransport):
         out, err = self.run_bzr('status', 'does-not-exist', retcode=3)
         self.assertContainsRe(err, r'do not exist.*does-not-exist')
 
+    def test_status_out_of_date(self):
+        """Simulate status of out-of-date tree after remote push"""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('a', 'foo\n')])
+        tree.lock_write()
+        try:
+            tree.add(['a'])
+            tree.commit('add test file')
+            # simulate what happens after a remote push
+            tree.set_last_revision("0")
+            out, err = self.run_bzr('status')
+            self.assertEqual("working tree is out of date, run 'bzr update'\n",
+                             err)
+        finally:
+            tree.unlock()
+
 
 class CheckoutStatus(BranchStatus):
 
@@ -208,25 +267,90 @@ class CheckoutStatus(BranchStatus):
 
 class TestStatus(TestCaseWithTransport):
 
-    def test_status(self):
+    def test_status_plain(self):
         self.run_bzr("init")
+
         self.build_tree(['hello.txt'])
         result = self.run_bzr("status")[0]
-        self.assert_("unknown:\n  hello.txt\n" in result, result)
+        self.assertContainsRe(result, "unknown:\n  hello.txt\n")
+
         self.run_bzr("add", "hello.txt")
         result = self.run_bzr("status")[0]
-        self.assert_("added:\n  hello.txt\n" in result, result)
+        self.assertContainsRe(result, "added:\n  hello.txt\n")
+
         self.run_bzr("commit", "-m", "added")
         result = self.run_bzr("status", "-r", "0..1")[0]
-        self.assert_("added:\n  hello.txt\n" in result, result)
+        self.assertContainsRe(result, "added:\n  hello.txt\n")
+
         self.build_tree(['world.txt'])
         result = self.run_bzr("status", "-r", "0")[0]
-        self.assert_("added:\n  hello.txt\n" \
-                     "unknown:\n  world.txt\n" in result, result)
-
+        self.assertContainsRe(result, "added:\n  hello.txt\n" \
+                                      "unknown:\n  world.txt\n")
         result2 = self.run_bzr("status", "-r", "0..")[0]
         self.assertEquals(result2, result)
 
+    def test_status_short(self):
+        self.run_bzr("init")
+
+        self.build_tree(['hello.txt'])
+        result = self.run_bzr("status","--short")[0]
+        self.assertContainsRe(result, "[?]   hello.txt\n")
+
+        self.run_bzr("add", "hello.txt")
+        result = self.run_bzr("status","--short")[0]
+        self.assertContainsRe(result, "[+]N  hello.txt\n")
+
+        self.run_bzr("commit", "-m", "added")
+        result = self.run_bzr("status", "--short", "-r", "0..1")[0]
+        self.assertContainsRe(result, "[+]N  hello.txt\n")
+
+        self.build_tree(['world.txt'])
+        result = self.run_bzr("status", "--short", "-r", "0")[0]
+        self.assertContainsRe(result, "[+]N  hello.txt\n" \
+                                      "[?]   world.txt\n")
+        result2 = self.run_bzr("status", "--short", "-r", "0..")[0]
+        self.assertEquals(result2, result)
+
+    def test_status_versioned(self):
+        self.run_bzr("init")
+
+        self.build_tree(['hello.txt'])
+        result = self.run_bzr("status", "--versioned")[0]
+        self.assertNotContainsRe(result, "unknown:\n  hello.txt\n")
+
+        self.run_bzr("add", "hello.txt")
+        result = self.run_bzr("status", "--versioned")[0]
+        self.assertContainsRe(result, "added:\n  hello.txt\n")
+
+        self.run_bzr("commit", "-m", "added")
+        result = self.run_bzr("status", "--versioned", "-r", "0..1")[0]
+        self.assertContainsRe(result, "added:\n  hello.txt\n")
+
+        self.build_tree(['world.txt'])
+        result = self.run_bzr("status", "--versioned", "-r", "0")[0]
+        self.assertContainsRe(result, "added:\n  hello.txt\n")
+        self.assertNotContainsRe(result, "unknown:\n  world.txt\n")
+        result2 = self.run_bzr("status", "--versioned", "-r", "0..")[0]
+        self.assertEquals(result2, result)
+
+    def assertStatusContains(self, pattern):
+        """Run status, and assert it contains the given pattern"""
+        result = self.run_bzr("status", "--short")[0]
+        self.assertContainsRe(result, pattern)
+
+    def test_kind_change_short(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['file'])
+        tree.add('file')
+        tree.commit('added file')
+        unlink('file')
+        self.build_tree(['file/'])
+        self.assertStatusContains('K  file => file/')
+        tree.rename_one('file', 'directory')
+        self.assertStatusContains('RK  file => directory/')
+        rmdir('directory')
+        self.assertStatusContains('RD  file => directory')
+
 
 class TestStatusEncodings(TestCaseWithTransport):
     
