@@ -1,4 +1,4 @@
-# Copyright (C) 2006 by Canonical Ltd
+# Copyright (C) 2006 Canonical Ltd
 # Authors: Robert Collins <robert.collins@canonical.com>
 # -*- coding: utf-8 -*-
 #
@@ -25,9 +25,9 @@ Specific tests for individual formats are in the tests/test_repository.py file
 rather than in tests/interrepository_implementations/*.py.
 """
 
-from bzrlib.repository import (_legacy_formats,
-                               InterRepositoryTestProviderAdapter,
-                               )
+from bzrlib.repository import (
+    InterRepositoryTestProviderAdapter,
+    )
                             
 from bzrlib.tests import (
                           adapt_modules,
