@@ -1,4 +1,4 @@
-# (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@ from bzrlib.errors import (FileExists,
                            NotBranchError,
                            )
 from bzrlib.inventory import Inventory
+import bzrlib.repofmt.weaverepo as weaverepo
 import bzrlib.repository as repository
 from bzrlib.revision import NULL_REVISION, Revision
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
@@ -97,16 +98,31 @@ def check_old_format_lock_error(repository_format):
     raise
 
 
+def check_repo_format_for_funky_id_on_win32(repo):
+    if (isinstance(repo, (weaverepo.AllInOneRepository,
+                          weaverepo.WeaveMetaDirRepository))
+        and sys.platform == 'win32'):
+            raise TestSkipped("funky chars does not permitted"
+                              " on this platform in repository"
+                              " %s" % repo.__class__.__name__)
+
+
 class TestInterRepository(TestCaseWithInterRepository):
 
     def test_interrepository_get_returns_correct_optimiser(self):
         # we assume the optimising code paths are triggered
         # by the type of the repo not the transport - at this point.
         # we may need to update this test if this changes.
-        source_repo = self.make_repository("source")
-        target_repo = self.make_to_repository("target")
-        interrepo = repository.InterRepository.get(source_repo, target_repo)
-        self.assertEqual(self.interrepo_class, interrepo.__class__)
+        #
+        # XXX: This code tests that we get an InterRepository when we try to
+        # convert between the two repositories that it wants to be tested with
+        # -- but that's not necessarily correct.  So for now this is disabled.
+        # mbp 20070206
+        ## source_repo = self.make_repository("source")
+        ## target_repo = self.make_to_repository("target")
+        ## interrepo = repository.InterRepository.get(source_repo, target_repo)
+        ## self.assertEqual(self.interrepo_class, interrepo.__class__)
+        pass
 
     def test_fetch(self):
         tree_a = self.make_branch_and_tree('a')
@@ -182,6 +198,9 @@ class TestInterRepository(TestCaseWithInterRepository):
 
     def test_fetch_funky_file_id(self):
         from_tree = self.make_branch_and_tree('tree')
+        if sys.platform == 'win32':
+            from_repo = from_tree.branch.repository
+            check_repo_format_for_funky_id_on_win32(from_repo)
         self.build_tree(['tree/filename'])
         from_tree.add('filename', 'funky-chars<>%&;"\'')
         from_tree.commit('commit filename')
@@ -191,6 +210,9 @@ class TestInterRepository(TestCaseWithInterRepository):
     def test_fetch_no_inventory_revision(self):
         """Old inventories lack revision_ids, so simulate this"""
         from_tree = self.make_branch_and_tree('tree')
+        if sys.platform == 'win32':
+            from_repo = from_tree.branch.repository
+            check_repo_format_for_funky_id_on_win32(from_repo)
         self.build_tree(['tree/filename'])
         from_tree.add('filename', 'funky-chars<>%&;"\'')
         from_tree.commit('commit filename')
