@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,9 +17,8 @@
 import os
 import sys
 
-from bzrlib.add import smart_add
 from bzrlib.builtins import merge
-from bzrlib.errors import IllegalPath
+from bzrlib.errors import IllegalPath, NonAsciiRevisionId
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
 from bzrlib.transform import TreeTransform
@@ -176,9 +175,10 @@ class TestFileIdInvolved(FileIdInvolvedBase):
     def fileids_altered_by_revision_ids(self, revision_ids):
         """This is a wrapper to strip TREE_ROOT if it occurs"""
         repo = self.branch.repository
+        root_id = self.branch.basis_tree().inventory.root.file_id
         result = repo.fileids_altered_by_revision_ids(revision_ids)
-        if 'TREE_ROOT' in result:
-            del result['TREE_ROOT']
+        if root_id in result:
+            del result[root_id]
         return result
 
     def test_fileids_altered_by_revision_ids(self):
@@ -222,6 +222,33 @@ class TestFileIdInvolved(FileIdInvolvedBase):
                 self.assertEquals(l1, l2)
 
 
+class TestFileIdInvolvedNonAscii(FileIdInvolvedBase):
+
+    def test_utf8_file_ids_and_revision_ids(self):
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a"])
+
+        file_id = u'a-f\xedle-id'.encode('utf8')
+        main_wt.add(['a'], [file_id])
+        revision_id = u'r\xe9v-a'.encode('utf8')
+        try:
+            main_wt.commit('a', rev_id=revision_id)
+        except NonAsciiRevisionId:
+            raise TestSkipped('non-ascii revision ids not supported by %s'
+                              % self.repository_format)
+
+        repo = main_wt.branch.repository
+        file_ids = repo.fileids_altered_by_revision_ids([revision_id])
+        root_id = main_wt.basis_tree().path2id('')
+        if root_id in file_ids:
+            self.assertEqual({file_id:set([revision_id]),
+                              root_id:set([revision_id])
+                             }, file_ids)
+        else:
+            self.assertEqual({file_id:set([revision_id])}, file_ids)
+
+
 class TestFileIdInvolvedSuperset(FileIdInvolvedBase):
 
     def setUp(self):
