@@ -26,7 +26,19 @@ from bzrlib import (
 
 class TestFileIds(tests.TestCase):
     """Test functions which generate file ids"""
-    
+
+    def assertGenFileId(self, regex, filename):
+        """gen_file_id should create a file id matching the regex.
+
+        The file id should be ascii, and should be an 8-bit string
+        """
+        file_id = generate_ids.gen_file_id(filename)
+        self.assertContainsRe(file_id, '^'+regex+'$')
+        # It should be a utf8 file_id, not a unicode one
+        self.assertIsInstance(file_id, str)
+        # gen_file_id should always return ascii file ids.
+        file_id.decode('ascii')
+
     def test_gen_file_id(self):
         gen_file_id = generate_ids.gen_file_id
 
@@ -43,7 +55,7 @@ class TestFileIds(tests.TestCase):
         # we remove unicode characters, and still don't end up with a 
         # hidden file id
         self.assertStartsWith(gen_file_id(u'\xe5\xb5.txt'), 'txt-')
-        
+
         # Our current method of generating unique ids adds 33 characters
         # plus an serial number (log10(N) characters)
         # to the end of the filename. We now restrict the filename portion to
@@ -60,6 +72,13 @@ class TestFileIds(tests.TestCase):
         self.assertStartsWith(fid, 'abcdefghijklmnopqrst-')
         self.failUnless(len(fid) < 60)
 
+    def test_file_ids_are_ascii(self):
+        tail = r'-\d{14}-[a-z0-9]{16}-\d+'
+        self.assertGenFileId('foo' + tail, 'foo')
+        self.assertGenFileId('foo' + tail, u'foo')
+        self.assertGenFileId('bar' + tail, u'bar')
+        self.assertGenFileId('br' + tail, u'b\xe5r')
+
     def test__next_id_suffix_sets_suffix(self):
         generate_ids._gen_file_id_suffix = None
         generate_ids._next_id_suffix()
@@ -102,6 +121,10 @@ class TestGenRevisionId(tests.TestCase):
         """gen_revision_id should create a revision id matching the regex"""
         revision_id = generate_ids.gen_revision_id(username, timestamp)
         self.assertMatchesRe(regex, revision_id)
+        # It should be a utf8 revision_id, not a unicode one
+        self.assertIsInstance(revision_id, str)
+        # gen_revision_id should always return ascii revision ids.
+        revision_id.decode('ascii')
 
     def test_timestamp(self):
         """passing a timestamp should cause it to be used"""
@@ -123,8 +146,19 @@ class TestGenRevisionId(tests.TestCase):
     def test_gen_revision_id_user(self):
         """If there is no email, fall back to the whole username"""
         tail = r'-\d{14}-[a-z0-9]{16}'
-        self.assertGenRevisionId('joe_bar' + tail,'Joe Bar')
+        self.assertGenRevisionId('joe_bar' + tail, 'Joe Bar')
         self.assertGenRevisionId('joebar' + tail, 'joebar')
         self.assertGenRevisionId('joe_br' + tail, u'Joe B\xe5r')
         self.assertGenRevisionId(r'joe_br_user\+joe_bar_foo-bar.com' + tail,
                                  u'Joe B\xe5r <user+Joe_Bar_Foo-Bar.com>')
+
+    def test_revision_ids_are_ascii(self):
+        """gen_revision_id should always return an ascii revision id."""
+        tail = r'-\d{14}-[a-z0-9]{16}'
+        self.assertGenRevisionId('joe_bar' + tail, 'Joe Bar')
+        self.assertGenRevisionId('joe_bar' + tail, u'Joe Bar')
+        self.assertGenRevisionId('joe@foo' + tail, u'Joe Bar <joe@foo>')
+        # We cheat a little with this one, because email-addresses shouldn't
+        # contain non-ascii characters, but generate_ids should strip them
+        # anyway.
+        self.assertGenRevisionId('joe@f' + tail, u'Joe Bar <joe@f\xb6>')
