@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 by Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,10 +56,13 @@ class _TestLockableFiles_mixin(object):
                               StringIO(unicode_string)
                               )
             self.lockable.put_utf8('bar', unicode_string)
-            self.assertEqual(unicode_string, 
+            self.assertEqual(unicode_string,
                              self.lockable.get_utf8('bar').read())
-            self.assertEqual(byte_string, 
+            self.assertEqual(byte_string,
                              self.lockable.get('bar').read())
+            self.lockable.put_bytes('raw', 'raw\xffbytes')
+            self.assertEqual('raw\xffbytes',
+                             self.lockable.get('raw').read())
         finally:
             self.lockable.unlock()
 
