@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,11 +27,15 @@ import warnings
 import bzrlib
 from bzrlib import (
     bzrdir,
+    errors,
     memorytree,
     osutils,
     repository,
+    symbol_versioning,
     )
 from bzrlib.progress import _BaseProgressBar
+from bzrlib.repofmt import weaverepo
+from bzrlib.symbol_versioning import zero_ten, zero_eleven
 from bzrlib.tests import (
                           ChrootedTestCase,
                           TestCase,
@@ -44,9 +48,6 @@ from bzrlib.tests import (
                           )
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.tests.TestUtil import _load_module_by_name
-import bzrlib.errors as errors
-from bzrlib import symbol_versioning
-from bzrlib.symbol_versioning import zero_ten, zero_eleven
 from bzrlib.trace import note
 from bzrlib.transport.memory import MemoryServer, MemoryTransport
 from bzrlib.version import _get_bzr_source_tree
@@ -353,7 +354,7 @@ class TestTreeProviderAdapter(TestCase):
             return_parameter,
             revision_tree_from_workingtree
             )
-        from bzrlib.workingtree import WorkingTreeFormat
+        from bzrlib.workingtree import WorkingTreeFormat, WorkingTreeFormat3
         input_test = TestTreeProviderAdapter(
             "test_adapted_tests")
         server1 = "a"
@@ -362,8 +363,10 @@ class TestTreeProviderAdapter(TestCase):
         adapter = TreeTestProviderAdapter(server1, server2, formats)
         suite = adapter.adapt(input_test)
         tests = list(iter(suite))
-        self.assertEqual(3, len(tests))
-        default_format = WorkingTreeFormat.get_default_format()
+        self.assertEqual(4, len(tests))
+        # this must match the default format setp up in
+        # TreeTestProviderAdapter.adapt
+        default_format = WorkingTreeFormat3
         self.assertEqual(tests[0].workingtree_format, formats[0][0])
         self.assertEqual(tests[0].bzrdir_format, formats[0][1])
         self.assertEqual(tests[0].transport_server, server1)
@@ -374,8 +377,9 @@ class TestTreeProviderAdapter(TestCase):
         self.assertEqual(tests[1].transport_server, server1)
         self.assertEqual(tests[1].transport_readonly_server, server2)
         self.assertEqual(tests[1].workingtree_to_test_tree, return_parameter)
-        self.assertEqual(tests[2].workingtree_format, default_format)
-        self.assertEqual(tests[2].bzrdir_format, default_format._matchingbzrdir)
+        self.assertIsInstance(tests[2].workingtree_format, default_format)
+        #self.assertEqual(tests[2].bzrdir_format,
+        #                 default_format._matchingbzrdir)
         self.assertEqual(tests[2].transport_server, server1)
         self.assertEqual(tests[2].transport_readonly_server, server2)
         self.assertEqual(tests[2].workingtree_to_test_tree,
@@ -409,24 +413,24 @@ class TestInterTreeProviderAdapter(TestCase):
         server2 = "b"
         format1 = WorkingTreeFormat2()
         format2 = WorkingTreeFormat3()
-        formats = [(str, format1, format2, False, True),
-            (int, format2, format1, False, True)]
+        formats = [(str, format1, format2, "converter1"),
+            (int, format2, format1, "converter2")]
         adapter = InterTreeTestProviderAdapter(server1, server2, formats)
         suite = adapter.adapt(input_test)
         tests = list(iter(suite))
         self.assertEqual(2, len(tests))
         self.assertEqual(tests[0].intertree_class, formats[0][0])
         self.assertEqual(tests[0].workingtree_format, formats[0][1])
-        self.assertEqual(tests[0].workingtree_to_test_tree, formats[0][2])
-        self.assertEqual(tests[0].workingtree_format_to, formats[0][3])
-        self.assertEqual(tests[0].workingtree_to_test_tree_to, formats[0][4])
+        self.assertEqual(tests[0].workingtree_format_to, formats[0][2])
+        self.assertEqual(tests[0].mutable_trees_to_test_trees, formats[0][3])
+        self.assertEqual(tests[0].workingtree_to_test_tree, return_parameter)
         self.assertEqual(tests[0].transport_server, server1)
         self.assertEqual(tests[0].transport_readonly_server, server2)
         self.assertEqual(tests[1].intertree_class, formats[1][0])
         self.assertEqual(tests[1].workingtree_format, formats[1][1])
-        self.assertEqual(tests[1].workingtree_to_test_tree, formats[1][2])
-        self.assertEqual(tests[1].workingtree_format_to, formats[1][3])
-        self.assertEqual(tests[1].workingtree_to_test_tree_to, formats[1][4])
+        self.assertEqual(tests[1].workingtree_format_to, formats[1][2])
+        self.assertEqual(tests[1].mutable_trees_to_test_trees, formats[1][3])
+        self.assertEqual(tests[1].workingtree_to_test_tree, return_parameter)
         self.assertEqual(tests[1].transport_server, server1)
         self.assertEqual(tests[1].transport_readonly_server, server2)
 
@@ -475,7 +479,7 @@ class TestTestCaseWithMemoryTransport(TestCaseWithMemoryTransport):
     def test_make_branch_and_memory_tree_with_format(self):
         """make_branch_and_memory_tree should accept a format option."""
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = repository.RepositoryFormat7()
+        format.repository_format = weaverepo.RepositoryFormat7()
         tree = self.make_branch_and_memory_tree('dir', format=format)
         # Guard against regression into MemoryTransport leaking
         # files to disk instead of keeping them in memory.
@@ -714,11 +718,53 @@ class TestRunner(TestCase):
         # skipping_test must be hidden in here so it's not run as a real test
         def skipping_test():
             raise TestSkipped('test intentionally skipped')
+
         runner = TextTestRunner(stream=self._log_file, keep_output=True)
         test = unittest.FunctionTestCase(skipping_test)
         result = self.run_test_runner(runner, test)
         self.assertTrue(result.wasSuccessful())
 
+    def test_skipped_from_setup(self):
+        class SkippedSetupTest(TestCase):
+
+            def setUp(self):
+                self.counter = 1
+                self.addCleanup(self.cleanup)
+                raise TestSkipped('skipped setup')
+
+            def test_skip(self):
+                self.fail('test reached')
+
+            def cleanup(self):
+                self.counter -= 1
+
+        runner = TextTestRunner(stream=self._log_file, keep_output=True)
+        test = SkippedSetupTest('test_skip')
+        result = self.run_test_runner(runner, test)
+        self.assertTrue(result.wasSuccessful())
+        # Check if cleanup was called the right number of times.
+        self.assertEqual(0, test.counter)
+
+    def test_skipped_from_test(self):
+        class SkippedTest(TestCase):
+
+            def setUp(self):
+                self.counter = 1
+                self.addCleanup(self.cleanup)
+
+            def test_skip(self):
+                raise TestSkipped('skipped test')
+
+            def cleanup(self):
+                self.counter -= 1
+
+        runner = TextTestRunner(stream=self._log_file, keep_output=True)
+        test = SkippedTest('test_skip')
+        result = self.run_test_runner(runner, test)
+        self.assertTrue(result.wasSuccessful())
+        # Check if cleanup was called the right number of times.
+        self.assertEqual(0, test.counter)
+
     def test_bench_history(self):
         # tests that the running the benchmark produces a history file
         # containing a timestamp and the revision id of the bzrlib source which
@@ -852,7 +898,12 @@ class TestTestCase(TestCase):
         self.assertContainsRe(
             output_stream.getvalue(),
             r"\d+ms/ +\d+ms\n$")
-        
+
+    def test_hooks_sanitised(self):
+        """The bzrlib hooks should be sanitised by setUp."""
+        self.assertEqual(bzrlib.branch.BranchHooks(),
+            bzrlib.branch.Branch.hooks)
+
     def test__gather_lsprof_in_benchmarks(self):
         """When _gather_lsprof_in_benchmarks is on, accumulate profile data.
         
