@@ -31,7 +31,7 @@ class TestamentSetup(TestCaseWithTransport):
 
     def setUp(self):
         super(TestamentSetup, self).setUp()
-        self.wt = self.make_branch_and_tree('.')
+        self.wt = self.make_branch_and_tree('.', format='dirstate-with-subtree')
         self.wt.set_root_id('TREE_ROT')
         b = self.b = self.wt.branch
         b.nick = "test branch"
@@ -281,7 +281,7 @@ parents:
 message:
   add files and directories
 inventory:
-  directory . TREE_ROT test@user-2 no
+  directory . TREE_ROT test@user-1 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
@@ -375,7 +375,7 @@ parents:
 message:
   revision with properties
 inventory:
-  directory . TREE_ROT test@user-3 no
+  directory . TREE_ROT test@user-1 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
@@ -444,7 +444,7 @@ parents:
 message:
   add symlink
 inventory:
-  directory . TREE_ROT test@user-3 no
+  directory . TREE_ROT test@user-1 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   symlink link link-id wibble/linktarget test@user-3 no
   directory src src-id test@user-2 no
@@ -510,7 +510,7 @@ parents:
 message:
   non-ascii commit \N{COPYRIGHT SIGN} me
 inventory:
-  directory . TREE_ROT test@user-3 no
+  directory . TREE_ROT test@user-1 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
