@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+# Copyright (C) 2004, 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,11 +21,13 @@ import stat
 from cStringIO import StringIO
 
 import bzrlib
+from bzrlib import urlutils
 from bzrlib.errors import (NoSuchFile, FileExists,
                            TransportNotPossible,
                            ConnectionError,
                            DependencyNotPresent,
                            UnsupportedProtocol,
+                           PathNotChild,
                            )
 from bzrlib.tests import TestCase, TestCaseInTempDir
 from bzrlib.transport import (_CoalescedOffset,
@@ -37,7 +39,11 @@ from bzrlib.transport import (_CoalescedOffset,
                               Transport,
                               )
 from bzrlib.transport.memory import MemoryTransport
-from bzrlib.transport.local import LocalTransport
+from bzrlib.transport.local import (LocalTransport,
+                                    EmulatedWin32LocalTransport)
+
+
+# TODO: Should possibly split transport-specific tests into their own files.
 
 
 class TestTransport(TestCase):
@@ -100,6 +106,17 @@ class TestTransport(TestCase):
         finally:
             _set_protocol_handlers(saved_handlers)
 
+    def test__combine_paths(self):
+        t = Transport('/')
+        self.assertEqual('/home/sarah/project/foo',
+                         t._combine_paths('/home/sarah', 'project/foo'))
+        self.assertEqual('/etc',
+                         t._combine_paths('/home/sarah', '../../etc'))
+        self.assertEqual('/etc',
+                         t._combine_paths('/home/sarah', '../../../etc'))
+        self.assertEqual('/etc',
+                         t._combine_paths('/home/sarah', '/etc'))
+
 
 class TestCoalesceOffsets(TestCase):
     
@@ -179,8 +196,9 @@ class TestMemoryTransport(TestCase):
         self.assertEqual("memory:///", transport.base)
         self.assertEqual("memory:///", transport.abspath('/'))
 
-    def test_relpath(self):
+    def test_abspath_of_relpath_starting_at_root(self):
         transport = MemoryTransport()
+        self.assertEqual("memory:///foo", transport.abspath('/foo'))
 
     def test_append_and_get(self):
         transport = MemoryTransport()
@@ -219,6 +237,16 @@ class TestMemoryTransport(TestCase):
         transport.append_bytes('foo', 'content')
         self.assertEquals(True, transport.has('foo'))
 
+    def test_list_dir(self):
+        transport = MemoryTransport()
+        transport.put_bytes('foo', 'content')
+        transport.mkdir('dir')
+        transport.put_bytes('dir/subfoo', 'content')
+        transport.put_bytes('dirlike', 'content')
+
+        self.assertEquals(['dir', 'dirlike', 'foo'], sorted(transport.list_dir('.')))
+        self.assertEquals(['subfoo'], sorted(transport.list_dir('dir')))
+
     def test_mkdir(self):
         transport = MemoryTransport()
         transport.mkdir('dir')
@@ -257,7 +285,89 @@ class TestMemoryTransport(TestCase):
         self.assertEqual(7, transport.stat('foo').st_size)
         self.assertEqual(6, transport.stat('bar').st_size)
 
-        
+
+class ChrootDecoratorTransportTest(TestCase):
+    """Chroot decoration specific tests."""
+
+    def test_construct(self):
+        from bzrlib.transport import chroot
+        transport = chroot.ChrootTransportDecorator('chroot+memory:///pathA/')
+        self.assertEqual('memory:///pathA/', transport.chroot_url)
+
+        transport = chroot.ChrootTransportDecorator(
+            'chroot+memory:///path/B', chroot='memory:///path/')
+        self.assertEqual('memory:///path/', transport.chroot_url)
+
+    def test_append_file(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.append_file, '/foo', None)
+
+    def test_append_bytes(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.append_bytes, '/foo', 'bytes')
+
+    def test_clone(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.clone, '/foo')
+
+    def test_delete(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.delete, '/foo')
+
+    def test_delete_tree(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.delete_tree, '/foo')
+
+    def test_get(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.get, '/foo')
+
+    def test_get_bytes(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.get_bytes, '/foo')
+
+    def test_has(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.has, '/foo')
+
+    def test_list_dir(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.list_dir, '/foo')
+
+    def test_lock_read(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.lock_read, '/foo')
+
+    def test_lock_write(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.lock_write, '/foo')
+
+    def test_mkdir(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.mkdir, '/foo')
+
+    def test_put_bytes(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.put_bytes, '/foo', 'bytes')
+
+    def test_put_file(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.put_file, '/foo', None)
+
+    def test_rename(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.rename, '/aaa', 'bbb')
+        self.assertRaises(PathNotChild, transport.rename, 'ccc', '/d')
+
+    def test_rmdir(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.rmdir, '/foo')
+
+    def test_stat(self):
+        transport = get_transport('chroot+memory:///foo/bar')
+        self.assertRaises(PathNotChild, transport.stat, '/foo')
+
+
 class ReadonlyDecoratorTransportTest(TestCase):
     """Readonly decoration specific tests."""
 
@@ -270,8 +380,8 @@ class ReadonlyDecoratorTransportTest(TestCase):
         self.assertEqual(True, transport.is_readonly())
 
     def test_http_parameters(self):
+        from bzrlib.tests.HttpServer import HttpServer
         import bzrlib.transport.readonly as readonly
-        from bzrlib.transport.http import HttpServer
         # connect to . via http which is not listable
         server = HttpServer()
         server.setUp()
@@ -305,7 +415,7 @@ class FakeNFSDecoratorTests(TestCaseInTempDir):
     def test_http_parameters(self):
         # the listable, should_cache and is_readonly parameters
         # are not changed by the fakenfs decorator
-        from bzrlib.transport.http import HttpServer
+        from bzrlib.tests.HttpServer import HttpServer
         # connect to . via http which is not listable
         server = HttpServer()
         server.setUp()
@@ -325,8 +435,8 @@ class FakeNFSDecoratorTests(TestCaseInTempDir):
         server = fakenfs.FakeNFSServer()
         server.setUp()
         try:
-            # the server should be a relpath localhost server
-            self.assertEqual(server.get_url(), 'fakenfs+.')
+            # the url should be decorated appropriately
+            self.assertStartsWith(server.get_url(), 'fakenfs+')
             # and we should be able to get a transport for it
             transport = get_transport(server.get_url())
             # which must be a FakeNFSTransportDecorator instance.
@@ -415,8 +525,45 @@ class TestTransportImplementation(TestCaseInTempDir):
         base_url = self._server.get_url()
         # try getting the transport via the regular interface:
         t = get_transport(base_url)
-        if not isinstance(t, self.transport_class): 
+        if not isinstance(t, self.transport_class):
             # we did not get the correct transport class type. Override the
             # regular connection behaviour by direct construction.
             t = self.transport_class(base_url)
         return t
+
+
+class TestLocalTransports(TestCase):
+
+    def test_get_transport_from_abspath(self):
+        here = os.path.abspath('.')
+        t = get_transport(here)
+        self.assertIsInstance(t, LocalTransport)
+        self.assertEquals(t.base, urlutils.local_path_to_url(here) + '/')
+
+    def test_get_transport_from_relpath(self):
+        here = os.path.abspath('.')
+        t = get_transport('.')
+        self.assertIsInstance(t, LocalTransport)
+        self.assertEquals(t.base, urlutils.local_path_to_url('.') + '/')
+
+    def test_get_transport_from_local_url(self):
+        here = os.path.abspath('.')
+        here_url = urlutils.local_path_to_url(here) + '/'
+        t = get_transport(here_url)
+        self.assertIsInstance(t, LocalTransport)
+        self.assertEquals(t.base, here_url)
+
+
+class TestWin32LocalTransport(TestCase):
+
+    def test_unc_clone_to_root(self):
+        # Win32 UNC path like \\HOST\path
+        # clone to root should stop at least at \\HOST part
+        # not on \\
+        t = EmulatedWin32LocalTransport('file://HOST/path/to/some/dir/')
+        for i in xrange(4):
+            t = t.clone('..')
+        self.assertEquals(t.base, 'file://HOST/')
+        # make sure we reach the root
+        t = t.clone('..')
+        self.assertEquals(t.base, 'file://HOST/')
