@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
+# Copyright (C) 2004, 2005, 2006 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,43 +38,16 @@ from bzrlib.osutils import getcwd
 from bzrlib.symbol_versioning import zero_eleven
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
 from bzrlib.tests.test_transport import TestTransportImplementation
-from bzrlib.transport import memory, smart
+from bzrlib.transport import memory, smart, chroot
 import bzrlib.transport
 
 
-def _append(fn, txt):
-    """Append the given text (file-like object) to the supplied filename."""
-    f = open(fn, 'ab')
-    try:
-        f.write(txt.read())
-    finally:
-        f.close()
-
-
 class TransportTests(TestTransportImplementation):
 
     def check_transport_contents(self, content, transport, relpath):
         """Check that transport.get(relpath).read() == content."""
         self.assertEqualDiff(content, transport.get(relpath).read())
 
-    def assertListRaises(self, excClass, func, *args, **kwargs):
-        """Fail unless excClass is raised when the iterator from func is used.
-        
-        Many transport functions can return generators this makes sure
-        to wrap them in a list() call to make sure the whole generator
-        is run, and that the proper exception is raised.
-        """
-        try:
-            list(func(*args, **kwargs))
-        except excClass:
-            return
-        else:
-            if getattr(excClass,'__name__', None) is not None:
-                excName = excClass.__name__
-            else:
-                excName = str(excClass)
-            raise self.failureException, "%s not raised" % excName
-
     def test_has(self):
         t = self.get_transport()
 
@@ -94,7 +67,8 @@ class TransportTests(TestTransportImplementation):
 
     def test_has_root_works(self):
         current_transport = self.get_transport()
-        # import pdb;pdb.set_trace()
+        if isinstance(current_transport, chroot.ChrootTransportDecorator):
+            raise TestSkipped("ChrootTransportDecorator disallows clone('..')")
         self.assertTrue(current_transport.has('/'))
         root = current_transport.clone('/')
         self.assertTrue(root.has(''))
@@ -776,6 +750,24 @@ class TransportTests(TestTransportImplementation):
         t.mkdir('adir/bdir')
         self.assertRaises(PathError, t.rmdir, 'adir')
 
+    def test_rmdir_empty_but_similar_prefix(self):
+        """rmdir does not get confused by sibling paths.
+        
+        A naive implementation of MemoryTransport would refuse to rmdir
+        ".bzr/branch" if there is a ".bzr/branch-format" directory, because it
+        uses "path.startswith(dir)" on all file paths to determine if directory
+        is empty.
+        """
+        t = self.get_transport()
+        if t.is_readonly():
+            return
+        t.mkdir('foo')
+        t.put_bytes('foo-bar', '')
+        t.mkdir('foo-baz')
+        t.rmdir('foo')
+        self.assertRaises((NoSuchFile, PathError), t.rmdir, 'foo')
+        self.failUnless(t.has('foo-bar'))
+
     def test_rename_dir_succeeds(self):
         t = self.get_transport()
         if t.is_readonly():
@@ -897,6 +889,8 @@ class TransportTests(TestTransportImplementation):
         except NotImplementedError:
             raise TestSkipped("Transport %s has no bogus URL support." %
                               self._server.__class__)
+        # This should be:  but SSH still connects on construction. No COOKIE!
+        # self.assertRaises((ConnectionError, NoSuchFile), t.get, '.bzr/branch')
         try:
             t = bzrlib.transport.get_transport(url)
             t.get('.bzr/branch')
@@ -959,23 +953,16 @@ class TransportTests(TestTransportImplementation):
             l.sort()
             return l
 
-        # SftpServer creates control files in the working directory
-        # so lets move down a directory to avoid those.
-        if not t.is_readonly():
-            t.mkdir('wd')
-        else:
-            os.mkdir('wd')
-        t = t.clone('wd')
-
         self.assertEqual([], sorted_list('.'))
         # c2 is precisely one letter longer than c here to test that
         # suffixing is not confused.
         # a%25b checks that quoting is done consistently across transports
         tree_names = ['a', 'a%25b', 'b', 'c/', 'c/d', 'c/e', 'c2/']
+
         if not t.is_readonly():
             self.build_tree(tree_names, transport=t)
         else:
-            self.build_tree(['wd/' + name for name in tree_names])
+            self.build_tree(tree_names)
 
         self.assertEqual(
             ['a', 'a%2525b', 'b', 'c', 'c2'], sorted_list('.'))
@@ -985,8 +972,8 @@ class TransportTests(TestTransportImplementation):
             t.delete('c/d')
             t.delete('b')
         else:
-            os.unlink('wd/c/d')
-            os.unlink('wd/b')
+            os.unlink('c/d')
+            os.unlink('b')
             
         self.assertEqual(['a', 'a%2525b', 'c', 'c2'], sorted_list('.'))
         self.assertEqual(['e'], sorted_list('c'))
@@ -1012,6 +999,8 @@ class TransportTests(TestTransportImplementation):
     def test_clone(self):
         # TODO: Test that clone moves up and down the filesystem
         t1 = self.get_transport()
+        if isinstance(t1, chroot.ChrootTransportDecorator):
+            raise TestSkipped("ChrootTransportDecorator disallows clone('..')")
 
         self.build_tree(['a', 'b/', 'b/c'], transport=t1)
 
@@ -1044,18 +1033,20 @@ class TransportTests(TestTransportImplementation):
 
     def test_clone_to_root(self):
         orig_transport = self.get_transport()
+        if isinstance(orig_transport, chroot.ChrootTransportDecorator):
+            raise TestSkipped("ChrootTransportDecorator disallows clone('..')")
         # Repeatedly go up to a parent directory until we're at the root
         # directory of this transport
         root_transport = orig_transport
         new_transport = root_transport.clone("..")
-        # as we are walking up directories, the path must be must be 
+        # as we are walking up directories, the path must be
         # growing less, except at the top
         self.assertTrue(len(new_transport.base) < len(root_transport.base)
             or new_transport.base == root_transport.base)
         while new_transport.base != root_transport.base:
             root_transport = new_transport
             new_transport = root_transport.clone("..")
-            # as we are walking up directories, the path must be must be 
+            # as we are walking up directories, the path must be
             # growing less, except at the top
             self.assertTrue(len(new_transport.base) < len(root_transport.base)
                 or new_transport.base == root_transport.base)
@@ -1072,6 +1063,8 @@ class TransportTests(TestTransportImplementation):
     def test_clone_from_root(self):
         """At the root, cloning to a simple dir should just do string append."""
         orig_transport = self.get_transport()
+        if isinstance(orig_transport, chroot.ChrootTransportDecorator):
+            raise TestSkipped("ChrootTransportDecorator disallows clone('/')")
         root_transport = orig_transport.clone('/')
         self.assertEqual(root_transport.base + '.bzr/',
             root_transport.clone('.bzr').base)
@@ -1092,6 +1085,8 @@ class TransportTests(TestTransportImplementation):
 
     def test_relpath_at_root(self):
         t = self.get_transport()
+        if isinstance(t, chroot.ChrootTransportDecorator):
+            raise TestSkipped("ChrootTransportDecorator disallows clone('..')")
         # clone all the way to the top
         new_transport = t.clone('..')
         while new_transport.base != t.base:
@@ -1107,6 +1102,8 @@ class TransportTests(TestTransportImplementation):
         # that have aliasing problems like symlinks should go in backend
         # specific test cases.
         transport = self.get_transport()
+        if isinstance(transport, chroot.ChrootTransportDecorator):
+            raise TestSkipped("ChrootTransportDecorator disallows clone('..')")
         
         self.assertEqual(transport.base + 'relpath',
                          transport.abspath('relpath'))
@@ -1117,6 +1114,9 @@ class TransportTests(TestTransportImplementation):
         # the abspath of "/" and "/foo/.." should result in the same location
         self.assertEqual(transport.abspath("/"), transport.abspath("/foo/.."))
 
+        self.assertEqual(transport.clone("/").abspath('foo'),
+                         transport.abspath("/foo"))
+
     def test_local_abspath(self):
         transport = self.get_transport()
         try:
@@ -1128,6 +1128,8 @@ class TransportTests(TestTransportImplementation):
 
     def test_abspath_at_root(self):
         t = self.get_transport()
+        if isinstance(t, chroot.ChrootTransportDecorator):
+            raise TestSkipped("ChrootTransportDecorator disallows clone('..')")
         # clone all the way to the top
         new_transport = t.clone('..')
         while new_transport.base != t.base:
@@ -1232,6 +1234,8 @@ class TransportTests(TestTransportImplementation):
         # check that our server (whatever it is) is accessable reliably
         # via get_transport and multiple connections share content.
         transport = self.get_transport()
+        if isinstance(transport, chroot.ChrootTransportDecorator):
+            raise TestSkipped("ChrootTransportDecorator disallows clone('..')")
         if transport.is_readonly():
             return
         transport.put_bytes('foo', 'bar')
@@ -1290,6 +1294,9 @@ class TransportTests(TestTransportImplementation):
         else:
             transport.put_bytes('a', '0123456789')
 
+        d = list(transport.readv('a', ((0, 1),)))
+        self.assertEqual(d[0], (0, '0'))
+
         d = list(transport.readv('a', ((0, 1), (1, 1), (3, 2), (9, 1))))
         self.assertEqual(d[0], (0, '0'))
         self.assertEqual(d[1], (1, '1'))
@@ -1309,19 +1316,14 @@ class TransportTests(TestTransportImplementation):
         self.assertEqual(d[2], (0, '0'))
         self.assertEqual(d[3], (3, '34'))
 
-    def test_get_smart_client(self):
-        """All transports must either give a smart client, or know they can't.
-
-        For some transports such as http this might depend on probing to see 
-        what's actually present on the other end.  (But we can adjust for that 
-        in the future.)
+    def test_get_smart_medium(self):
+        """All transports must either give a smart medium, or know they can't.
         """
         transport = self.get_transport()
         try:
-            client = transport.get_smart_client()
-            # XXX: should be a more general class
-            self.assertIsInstance(client, smart.SmartStreamClient)
-        except NoSmartServer:
+            medium = transport.get_smart_medium()
+            self.assertIsInstance(medium, smart.SmartClientMedium)
+        except errors.NoSmartMedium:
             # as long as we got it we're fine
             pass
 
@@ -1334,10 +1336,12 @@ class TransportTests(TestTransportImplementation):
 
         # This is intentionally reading off the end of the file
         # since we are sure that it cannot get there
-        self.assertListRaises((errors.ShortReadvError, AssertionError),
+        self.assertListRaises((errors.ShortReadvError, errors.InvalidRange,
+                               # Can be raised by paramiko
+                               AssertionError),
                               transport.readv, 'a', [(1,1), (8,10)])
 
         # This is trying to seek past the end of the file, it should
         # also raise a special error
-        self.assertListRaises(errors.ShortReadvError,
+        self.assertListRaises((errors.ShortReadvError, errors.InvalidRange),
                               transport.readv, 'a', [(12,2)])
