@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -58,7 +58,11 @@ from bzrlib.symbol_versioning import (
         zero_eight,
         zero_eleven,
         )
-from bzrlib.trace import mutter, warning
+from bzrlib.trace import (
+    note,
+    mutter,
+    warning,
+    )
 
 # {prefix: [transport_classes]}
 # Transports are inserted onto the list LIFO and tried in order; as a result
@@ -93,6 +97,8 @@ def register_lazy_transport(scheme, module, classname):
     intended to be used when the implementation depends on an external
     implementation that may not be present.  If any other error is raised, it
     propagates up and the attempt to open the url fails.
+
+    :param scheme: The url scheme part, eg "ftp://"
     """
     # TODO: If no implementation of a protocol is available because of missing
     # dependencies, we should perhaps show the message about what dependency
@@ -131,7 +137,7 @@ def _get_transport_modules():
         for factory in factory_list:
             if factory.__module__ == "bzrlib.transport":
                 # this is a lazy load transport, because no real ones
-                # are directlry in bzrlib.transport
+                # are directly in bzrlib.transport
                 modules.add(factory.module)
             else:
                 modules.add(factory.__module__)
@@ -1004,12 +1010,9 @@ def get_transport(base):
 
     base is either a URL or a directory name.  
     """
-    # TODO: give a better error if base looks like a url but there's no
-    # handler for the scheme?
     global _protocol_handlers
     if base is None:
         base = '.'
-
     last_err = None
 
     def convert_path_to_url(base, error_str):
@@ -1045,6 +1048,48 @@ def get_transport(base):
     return _try_transport_factories(base, _protocol_handlers[None])[0]
 
 
+def do_catching_redirections(action, transport, redirected):
+    """Execute an action with given transport catching redirections.
+
+    This is a facility provided for callers needing to follow redirections
+    silently. The silence is relative: it is the caller responsability to
+    inform the user about each redirection or only inform the user of a user
+    via the exception parameter.
+
+    :param action: A callable, what the caller want to do while catching
+                  redirections.
+    :param transport: The initial transport used.
+    :param redirected: A callable receiving the redirected transport and the 
+                  RedirectRequested exception.
+
+    :return: Whatever 'action' returns
+    """
+    MAX_REDIRECTIONS = 8
+
+    # If a loop occurs, there is little we can do. So we don't try to detect
+    # them, just getting out if too much redirections occurs. The solution
+    # is outside: where the loop is defined.
+    for redirections in range(MAX_REDIRECTIONS):
+        try:
+            return action(transport)
+        except errors.RedirectRequested, e:
+            redirection_notice = '%s is%s redirected to %s' % (
+                e.source, e.permanently, e.target)
+            transport = redirected(transport, e, redirection_notice)
+    else:
+        # Loop exited without resolving redirect ? Either the
+        # user has kept a very very very old reference or a loop
+        # occurred in the redirections.  Nothing we can cure here:
+        # tell the user. Note that as the user has been informed
+        # about each redirection (it is the caller responsibility
+        # to do that in redirected via the provided
+        # redirection_notice). The caller may provide more
+        # information if needed (like what file or directory we
+        # were trying to act upon when the redirection loop
+        # occurred).
+        raise errors.TooManyRedirections
+
+
 def _try_transport_factories(base, factory_list):
     last_err = None
     for factory in factory_list:
@@ -1166,6 +1211,7 @@ class TransportLogger(object):
 register_lazy_transport(None, 'bzrlib.transport.local', 'LocalTransport')
 register_lazy_transport('file://', 'bzrlib.transport.local', 'LocalTransport')
 register_lazy_transport('sftp://', 'bzrlib.transport.sftp', 'SFTPTransport')
+# Decorated http transport
 register_lazy_transport('http+urllib://', 'bzrlib.transport.http._urllib',
                         'HttpTransport_urllib')
 register_lazy_transport('https+urllib://', 'bzrlib.transport.http._urllib',
@@ -1174,6 +1220,7 @@ register_lazy_transport('http+pycurl://', 'bzrlib.transport.http._pycurl',
                         'PyCurlTransport')
 register_lazy_transport('https+pycurl://', 'bzrlib.transport.http._pycurl',
                         'PyCurlTransport')
+# Default http transports (last declared wins (if it can be imported))
 register_lazy_transport('http://', 'bzrlib.transport.http._urllib',
                         'HttpTransport_urllib')
 register_lazy_transport('https://', 'bzrlib.transport.http._urllib',
