@@ -23,19 +23,35 @@
 # whether we expect a particular file will be modified after it's committed.
 # It's probably safer to just always revalidate.  mbp 20060321
 
+# TODO: Some refactoring could be done to avoid the strange idiom
+# used to capture data and headers while setting up the request
+# (and having to pass 'header' to _curl_perform to handle
+# redirections) . This could be achieved by creating a
+# specialized Curl object and returning code, headers and data
+# from _curl_perform.  Not done because we may deprecate pycurl in the
+# future -- vila 20070212
+
 import os
 from cStringIO import StringIO
+import sys
 
-from bzrlib import errors
+from bzrlib import (
+    errors,
+    __version__ as bzrlib_version,
+    )
 import bzrlib
-from bzrlib.errors import (TransportNotPossible, NoSuchFile,
-                           TransportError, ConnectionError,
+from bzrlib.errors import (NoSuchFile,
+                           ConnectionError,
                            DependencyNotPresent)
 from bzrlib.trace import mutter
 from bzrlib.transport import register_urlparse_netloc_protocol
-from bzrlib.transport.http import (HttpTransportBase, HttpServer,
-                                   _extract_headers,
-                                   response, _pycurl_errors)
+from bzrlib.transport.http import (
+    ca_bundle,
+    _extract_headers,
+    HttpTransportBase,
+    _pycurl_errors,
+    response,
+    )
 
 try:
     import pycurl
@@ -67,20 +83,24 @@ class PyCurlTransport(HttpTransportBase):
 
     PyCurl is a Python binding to the C "curl" multiprotocol client.
 
-    This transport can be significantly faster than the builtin Python client. 
-    Advantages include: DNS caching, connection keepalive, and ability to 
-    set headers to allow caching.
+    This transport can be significantly faster than the builtin
+    Python client.  Advantages include: DNS caching.
     """
 
     def __init__(self, base, from_transport=None):
         super(PyCurlTransport, self).__init__(base)
+        if base.startswith('https'):
+            # Check availability of https into pycurl supported
+            # protocols
+            supported = pycurl.version_info()[8]
+            if 'https' not in supported:
+                raise DependencyNotPresent('pycurl', 'no https support')
+        self.cabundle = ca_bundle.get_ca_path()
         if from_transport is not None:
-            self._base_curl = from_transport._base_curl
-            self._range_curl = from_transport._range_curl
+            self._curl = from_transport._curl
         else:
             mutter('using pycurl %s' % pycurl.version)
-            self._base_curl = pycurl.Curl()
-            self._range_curl = pycurl.Curl()
+            self._curl = pycurl.Curl()
 
     def should_cache(self):
         """Return True if the data pulled across should be cached locally.
@@ -91,30 +111,47 @@ class PyCurlTransport(HttpTransportBase):
         """See Transport.has()"""
         # We set NO BODY=0 in _get_full, so it should be safe
         # to re-use the non-range curl object
-        curl = self._base_curl
+        curl = self._curl
         abspath = self._real_abspath(relpath)
         curl.setopt(pycurl.URL, abspath)
         self._set_curl_options(curl)
+        curl.setopt(pycurl.HTTPGET, 1)
         # don't want the body - ie just do a HEAD request
         # This means "NO BODY" not 'nobody'
         curl.setopt(pycurl.NOBODY, 1)
-        self._curl_perform(curl)
+        # But we need headers to handle redirections
+        header = StringIO()
+        curl.setopt(pycurl.HEADERFUNCTION, header.write)
+        # In some erroneous cases, pycurl will emit text on
+        # stdout if we don't catch it (see InvalidStatus tests
+        # for one such occurrence).
+        blackhole = StringIO()
+        curl.setopt(pycurl.WRITEFUNCTION, blackhole.write)
+        self._curl_perform(curl, header)
         code = curl.getinfo(pycurl.HTTP_CODE)
         if code == 404: # not found
             return False
-        elif code in (200, 302): # "ok", "found"
+        elif code == 200: # "ok"
             return True
         else:
             self._raise_curl_http_error(curl)
-        
+
     def _get(self, relpath, ranges, tail_amount=0):
         # This just switches based on the type of request
         if ranges is not None or tail_amount not in (0, None):
             return self._get_ranged(relpath, ranges, tail_amount=tail_amount)
         else:
             return self._get_full(relpath)
-    
+
     def _setup_get_request(self, curl, relpath):
+        # Make sure we do a GET request. versions > 7.14.1 also set the
+        # NO BODY flag, but we'll do it ourselves in case it is an older
+        # pycurl version
+        curl.setopt(pycurl.NOBODY, 0)
+        curl.setopt(pycurl.HTTPGET, 1)
+        return self._setup_request(curl, relpath)
+
+    def _setup_request(self, curl, relpath):
         """Do the common setup stuff for making a request
 
         :param curl: The curl object to place the request on
@@ -127,11 +164,6 @@ class PyCurlTransport(HttpTransportBase):
         abspath = self._real_abspath(relpath)
         curl.setopt(pycurl.URL, abspath)
         self._set_curl_options(curl)
-        # Make sure we do a GET request. versions > 7.14.1 also set the
-        # NO BODY flag, but we'll do it ourselves in case it is an older
-        # pycurl version
-        curl.setopt(pycurl.NOBODY, 0)
-        curl.setopt(pycurl.HTTPGET, 1)
 
         data = StringIO()
         header = StringIO()
@@ -142,9 +174,9 @@ class PyCurlTransport(HttpTransportBase):
 
     def _get_full(self, relpath):
         """Make a request for the entire file"""
-        curl = self._base_curl
+        curl = self._curl
         abspath, data, header = self._setup_get_request(curl, relpath)
-        self._curl_perform(curl)
+        self._curl_perform(curl, header)
 
         code = curl.getinfo(pycurl.HTTP_CODE)
         data.seek(0)
@@ -152,25 +184,24 @@ class PyCurlTransport(HttpTransportBase):
         if code == 404:
             raise NoSuchFile(abspath)
         if code != 200:
-            self._raise_curl_http_error(curl, 'expected 200 or 404 for full response.')
+            self._raise_curl_http_error(
+                curl, 'expected 200 or 404 for full response.')
 
         return code, data
 
     def _get_ranged(self, relpath, ranges, tail_amount):
         """Make a request for just part of the file."""
-        # We would like to re-use the same curl object for 
-        # full requests and partial requests
-        # Documentation says 'Pass in NULL to disable the use of ranges'
-        # None is the closest we have, but at least with pycurl 7.13.1
-        # It raises an 'invalid arguments' response
-        # curl.setopt(pycurl.RANGE, None)
-        # curl.unsetopt(pycurl.RANGE) doesn't support the RANGE parameter
-        # So instead we hack around this by using a separate objects
-        curl = self._range_curl
+        curl = self._curl
         abspath, data, header = self._setup_get_request(curl, relpath)
 
-        curl.setopt(pycurl.RANGE, self.range_header(ranges, tail_amount))
-        self._curl_perform(curl)
+        range_header = self.attempted_range_header(ranges, tail_amount)
+        if range_header is None:
+            # Forget ranges, the server can't handle them
+            return self._get_full(relpath)
+
+        self._curl_perform(curl, header,
+                           ['Range: bytes=%s'
+                            % self.range_header(ranges, tail_amount)])
         data.seek(0)
 
         code = curl.getinfo(pycurl.HTTP_CODE)
@@ -179,66 +210,95 @@ class PyCurlTransport(HttpTransportBase):
         # handle_response will raise NoSuchFile, etc based on the response code
         return code, response.handle_response(abspath, code, headers, data)
 
-    def _raise_curl_connection_error(self, curl):
-        curl_errno = curl.getinfo(pycurl.OS_ERRNO)
-        url = curl.getinfo(pycurl.EFFECTIVE_URL)
-        raise ConnectionError('curl connection error (%s) on %s'
-                              % (os.strerror(curl_errno), url))
+    def _post(self, body_bytes):
+        fake_file = StringIO(body_bytes)
+        curl = self._curl
+        # Other places that use _base_curl for GET requests explicitly set
+        # HTTPGET, so it should be safe to re-use the same object for both GETs
+        # and POSTs.
+        curl.setopt(pycurl.POST, 1)
+        curl.setopt(pycurl.POSTFIELDSIZE, len(body_bytes))
+        curl.setopt(pycurl.READFUNCTION, fake_file.read)
+        abspath, data, header = self._setup_request(curl, '.bzr/smart')
+        # We override the Expect: header so that pycurl will send the POST
+        # body immediately.
+        self._curl_perform(curl, header, ['Expect: '])
+        data.seek(0)
+        code = curl.getinfo(pycurl.HTTP_CODE)
+        headers = _extract_headers(header.getvalue(), abspath)
+        return code, response.handle_response(abspath, code, headers, data)
 
     def _raise_curl_http_error(self, curl, info=None):
         code = curl.getinfo(pycurl.HTTP_CODE)
         url = curl.getinfo(pycurl.EFFECTIVE_URL)
-        if info is None:
-            msg = ''
+        # Some error codes can be handled the same way for all
+        # requests
+        if code == 403:
+            raise errors.TransportError(
+                'Server refuses to fullfil the request for: %s' % url)
         else:
-            msg = ': ' + info
-        raise errors.InvalidHttpResponse(url, 'Unable to handle http code %d%s'
-                                              % (code,msg))
+            if info is None:
+                msg = ''
+            else:
+                msg = ': ' + info
+            raise errors.InvalidHttpResponse(
+                url, 'Unable to handle http code %d%s' % (code,msg))
 
     def _set_curl_options(self, curl):
         """Set options for all requests"""
-        # There's no way in http/1.0 to say "must revalidate"; we don't want
-        # to force it to always retrieve.  so just turn off the default Pragma
-        # provided by Curl.
-        headers = ['Cache-control: max-age=0',
-                   'Pragma: no-cache',
-                   'Connection: Keep-Alive']
         ## curl.setopt(pycurl.VERBOSE, 1)
         # TODO: maybe include a summary of the pycurl version
         ua_str = 'bzr/%s (pycurl)' % (bzrlib.__version__,)
         curl.setopt(pycurl.USERAGENT, ua_str)
-        curl.setopt(pycurl.HTTPHEADER, headers)
-        curl.setopt(pycurl.FOLLOWLOCATION, 1) # follow redirect responses
+        if self.cabundle:
+            curl.setopt(pycurl.CAINFO, self.cabundle)
 
-    def _curl_perform(self, curl):
+    def _curl_perform(self, curl, header, more_headers=[]):
         """Perform curl operation and translate exceptions."""
         try:
+            # There's no way in http/1.0 to say "must
+            # revalidate"; we don't want to force it to always
+            # retrieve.  so just turn off the default Pragma
+            # provided by Curl.
+            headers = ['Cache-control: max-age=0',
+                       'Pragma: no-cache',
+                       'Connection: Keep-Alive']
+            curl.setopt(pycurl.HTTPHEADER, headers + more_headers)
             curl.perform()
         except pycurl.error, e:
-            # XXX: There seem to be no symbolic constants for these values.
             url = curl.getinfo(pycurl.EFFECTIVE_URL)
             mutter('got pycurl error: %s, %s, %s, url: %s ',
                     e[0], _pycurl_errors.errorcode[e[0]], e, url)
             if e[0] in (_pycurl_errors.CURLE_COULDNT_RESOLVE_HOST,
-                        _pycurl_errors.CURLE_COULDNT_CONNECT):
-                self._raise_curl_connection_error(curl)
-            # jam 20060713 The code didn't use to re-raise the exception here
+                        _pycurl_errors.CURLE_COULDNT_CONNECT,
+                        _pycurl_errors.CURLE_GOT_NOTHING,
+                        _pycurl_errors.CURLE_COULDNT_RESOLVE_PROXY):
+                raise ConnectionError('curl connection error (%s)\non %s'
+                              % (e[1], url))
+            elif e[0] == _pycurl_errors.CURLE_PARTIAL_FILE:
+                # Pycurl itself has detected a short read.  We do
+                # not have all the information for the
+                # ShortReadvError, but that should be enough
+                raise errors.ShortReadvError(url,
+                                             offset='unknown', length='unknown',
+                                             actual='unknown',
+                                             extra='Server aborted the request')
+            # jam 20060713 The code didn't use to re-raise the exception here,
             # but that seemed bogus
             raise
-
-
-class HttpServer_PyCurl(HttpServer):
-    """Subclass of HttpServer that gives http+pycurl urls.
-
-    This is for use in testing: connections to this server will always go
-    through pycurl where possible.
-    """
-
-    # urls returned by this server should require the pycurl client impl
-    _url_protocol = 'http+pycurl'
+        code = curl.getinfo(pycurl.HTTP_CODE)
+        if code in (301, 302, 303, 307):
+            url = curl.getinfo(pycurl.EFFECTIVE_URL)
+            headers = _extract_headers(header.getvalue(), url)
+            redirected_to = headers['Location']
+            raise errors.RedirectRequested(url,
+                                           redirected_to,
+                                           is_permament=(code == 301),
+                                           qual_proto=self._qualified_proto)
 
 
 def get_test_permutations():
     """Return the permutations to be used in testing."""
+    from bzrlib.tests.HttpServer import HttpServer_PyCurl
     return [(PyCurlTransport, HttpServer_PyCurl),
             ]
