@@ -19,12 +19,22 @@
 """Text UI, write output to the console.
 """
 
-import getpass
 import sys
 
-import bzrlib.progress
-from bzrlib.symbol_versioning import (deprecated_method, 
-        zero_eight)
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import getpass
+
+from bzrlib import (
+    progress,
+    osutils,
+    )
+""")
+
+from bzrlib.symbol_versioning import (
+    deprecated_method,
+    zero_eight,
+    )
 from bzrlib.ui import CLIUIFactory
 
 
@@ -54,30 +64,14 @@ class TextUIFactory(CLIUIFactory):
 
     def prompt(self, prompt):
         """Emit prompt on the CLI."""
-        self.stdout.write(prompt + "? [y/n]:")
+        self.stdout.write(prompt)
         
     @deprecated_method(zero_eight)
     def progress_bar(self):
         """See UIFactory.nested_progress_bar()."""
         # this in turn is abstract, and creates either a tty or dots
         # bar depending on what we think of the terminal
-        return bzrlib.progress.ProgressBar()
-
-    def get_password(self, prompt='', **kwargs):
-        """Prompt the user for a password.
-
-        :param prompt: The prompt to present the user
-        :param kwargs: Arguments which will be expanded into the prompt.
-                       This lets front ends display different things if
-                       they so choose.
-        :return: The password string, return None if the user 
-                 canceled the request.
-        """
-        prompt = (prompt % kwargs).encode(sys.stdout.encoding, 'replace')
-        prompt += ': '
-        # There's currently no way to say 'i decline to enter a password'
-        # as opposed to 'my password is empty' -- does it matter?
-        return getpass.getpass(prompt)
+        return progress.ProgressBar()
 
     def nested_progress_bar(self):
         """Return a nested progress bar.
@@ -86,7 +80,7 @@ class TextUIFactory(CLIUIFactory):
         may return a tty or dots bar depending on the terminal.
         """
         if self._progress_bar_stack is None:
-            self._progress_bar_stack = bzrlib.progress.ProgressBarStack(
+            self._progress_bar_stack = progress.ProgressBarStack(
                 klass=self._bar_type)
         return self._progress_bar_stack.get_nested()
 
