@@ -75,6 +75,9 @@ import sha
 import time
 import warnings
 
+from bzrlib import (
+    progress,
+    )
 from bzrlib.trace import mutter
 from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
         RevisionAlreadyPresent,
@@ -244,6 +247,7 @@ class Weave(VersionedFile):
 
     def _lookup(self, name):
         """Convert symbolic version name to index."""
+        self.check_not_reserved_id(name)
         try:
             return self._name_map[name]
         except KeyError:
@@ -664,7 +668,8 @@ class Weave(VersionedFile):
         """_walk has become visit, a supported api."""
         return self._walk_internal()
 
-    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+    def iter_lines_added_or_present_in_versions(self, version_ids=None,
+                                                pb=None):
         """See VersionedFile.iter_lines_added_or_present_in_versions()."""
         if version_ids is None:
             version_ids = self.versions()
@@ -1116,6 +1121,7 @@ class WeaveFile(Weave):
 
     def _add_lines(self, version_id, parents, lines, parent_texts):
         """Add a version and save the weave."""
+        self.check_not_reserved_id(version_id)
         result = super(WeaveFile, self)._add_lines(version_id, parents, lines,
                                                    parent_texts)
         self._save()
